/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "audio.h"

#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "soundcard.h"
#include "mixer.h"
#include "record_gui.h"
#include "error.h"
#include "audio_gui.h"
#include "mark_gui.h"

static short buf[4096];
static short buf2[4096];

static int mode;
static int opened = 0;
static unsigned int event_id;
static unsigned int event_id2;

static int nb_in, nb_out;

static void play_audio_ready(void)
{
  int ret;

  if (mode == PLAY_SOUND)
    ret = main_mixer->mix(main_mixer, buf, 1024);
  else
    ret = main_mixer->mix_mono(main_mixer, buf, 2048);

  if (ret == -1) {
    close_audio();
    return;
  }

  if (!play_sound(buf, 4096))
    close_audio();

  play_mark_set(main_mixer->next_sample);

  nb_out++;
}

static void record_audio_ready(void)
{
  int ret;
#if 0
extern int record_ready_buffers();
int n = record_ready_buffers();
if (n<7) return;
#endif

  if (!record_sound(buf2, 4096)) {
    close_audio();
    error_record();
    return;
  }

  nb_in++;

  if ((ret=write(record_file_fd, buf2, 4096)) != 4096) {
    int serr = errno;
    close_audio();
    error_record();
    reset_error();
    add_error_char("Record\n");
    add_error_char("Saving to file: ");
    if (ret == -1) {
      add_error_char(strerror(serr));
      add_error_char("\nHint: ");
    }
    add_error_char("hard disk may be full, try to make some room.\n");
    add_error_char("(Save your project first, it's safer.)\n");
    add_error_char("(But if your disk is full, you may be unable to save your project, so...)\n");
    display_error();
    return;
  }
}

static void adjust_audio_ready(void)
{
  extern short adjust_audio_buf[2048];
  extern void adjust_input_update(void);

  if (!record_sound(adjust_audio_buf, 4096)) {
    close_audio();
    return;
  }

  adjust_input_update();
}

int open_audio_play(int requested_audio_mode)
{
  if (opened) {
    reset_error();
    add_error_char("Audio is already in use\n");
    display_error();
    return -2;
  }

  if (requested_audio_mode == PLAY_SOUND) {
    main_mixer->reset_effects(main_mixer);
    if (!init_sound(44100, 16, 1, 12, PLAY_SOUND))
      return -1;
    event_id = audio_gui_register_callback(PLAY_SOUND, play_audio_ready);
  } else if (requested_audio_mode == RECORD_ADJUST) {
    if (!init_sound(44100, 16, 0, 12, RECORD_SOUND))
      return -1;
    event_id = audio_gui_register_callback(RECORD_SOUND, adjust_audio_ready);
  } else if (requested_audio_mode == RECORD_SOUND) {
    if (!init_sound(44100, 16, 0, 12, RECORD_SOUND))
      return -1;
    event_id = audio_gui_register_callback(RECORD_SOUND, record_audio_ready);
  }  else {
    main_mixer->reset_effects(main_mixer);
    if (!init_sound(44100, 16, 0, 12, PLAY_AND_RECORD_SOUND))
      return -1;
    event_id = audio_gui_register_callback(PLAY_SOUND, play_audio_ready);
    event_id2 = audio_gui_register_callback(RECORD_SOUND, record_audio_ready);
  }

  mode = requested_audio_mode;

  nb_in = nb_out = 0;

  opened = 1;

  return 0;
}

void close_audio(void)
{
  if (!opened)
    return;

//  sound_stats();
  close_sound();

  audio_gui_unregister_callback(event_id);

  if (mode == PLAY_AND_RECORD_SOUND)
    audio_gui_unregister_callback(event_id2);

  play_mark_off();

  opened = 0;
}
