/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "channel.h"

char *type_name[] = {
  0,
  "input but you should not read this",
  "position",
  "panning",
  "gain",
  "compressor"
};

#include <stdlib.h>

#include "pan.h"
#include "vol.h"
#include "pos.h"
#include "sc1.h"
#include "input.h"
#include "error.h"

channel *new_channel(int type, channel *sub)
{
  switch (type) {
    case CHANNEL_TYPE_POS:
      return (channel *)new_pos(sub);

    case CHANNEL_TYPE_PAN:
      return (channel *)new_pan(sub);

    case CHANNEL_TYPE_VOL:
      return (channel *)new_vol(sub);

    case CHANNEL_TYPE_SC1:
      return (channel *)new_sc1(sub);

    default:
      reset_error();
      add_error_char("Unhandled type (");
      add_error_int(type);
      add_error_char(") in new_channel, contact the authors.\n");
      display_error();
      return 0;
  }
}

char *channel_type_to_char(int t)
{
  if (t <= 0 || t > 5)
    return UNKNOWN_CHANNEL_TYPE;

  return type_name[t];
}

void delete_channel(channel *n)
{
  if (n->c.gui_on)
    n->c.close_gui(n);

  switch(n->c.type) {
    case CHANNEL_TYPE_INPUT:
      delete_input((input *)n);
      break;
    case CHANNEL_TYPE_POS:
      delete_pos((pos *)n);
      break;
    case CHANNEL_TYPE_PAN:
      delete_pan((pan *)n);
      break;
    case CHANNEL_TYPE_VOL:
      delete_vol((vol *)n);
      break;
    case CHANNEL_TYPE_SC1:
      delete_sc1((sc1 *)n);
      break;
    case CHANNEL_TYPE_MIXER:
      /* nothing to do */
      break;
    default:
      reset_error();
      add_error_char(__FILE__);
      add_error_char(":");
      add_error_int(__LINE__);
      add_error_char(":");
      add_error_char(__FUNCTION__);
      add_error_char(": there is a big problem,\n");
      add_error_char("save your data (if you can) and contact the authors.\n");
      display_error();
      free(n);
  }
}

void delete_channel_and_sub(channel *c)
{
  channel *n;

  while (c) {
    n = c;
    c = c->c.subchannel;
    delete_channel(n);
  }
}

channel *channel_getsub(channel *c, int pos)
{
  while (c && pos) {
    pos--;
    c = c->c.subchannel;
  }

  return c;
}

channel *channel_remove(channel *a, channel *b)
{
  channel *next;
  channel *cur;

  if (a == b)
    return a->c.subchannel;

  cur = a;
  next = cur->c.subchannel;

  while (next && next != b) {
    cur = next;
    next = cur->c.subchannel;
  }

  if (!next)
    return 0;

  cur->c.subchannel = next->c.subchannel;

  return a;
}

channel *channel_insert(channel *a, channel *b, int pos)
{
  channel *cur;

  if (pos == 0) {
    b->c.subchannel = a;
    return b;
  }

  cur = a;
  pos--;

  while (pos && cur) {
    cur = cur->c.subchannel;
    pos--;
  }

  if (!cur)
    return 0;

  b->c.subchannel = cur->c.subchannel;
  cur->c.subchannel = b;

  return a;
}
