/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#ifndef _CHANNEL_H_
#define _CHANNEL_H_

/* channel types */
#define CHANNEL_NO_TYPE		0
#define CHANNEL_TYPE_INPUT	1
#define CHANNEL_TYPE_POS	2
#define CHANNEL_TYPE_PAN	3
#define CHANNEL_TYPE_VOL	4
#define CHANNEL_TYPE_SC1	5
#define CHANNEL_TYPE_MIXER	0xffff

#define FIRST_EFFECTABLE_CHANNEL 3
#define UNKNOWN_CHANNEL_TYPE "Unknown channel type"

/* input channels types */
#define RAW_TYPE_16B_LITTLE_ENDIAN_MONO			0
#define RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_LEFT		1
#define RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_RIGHT		2
#define RAW_TYPE_FLOAT_MONO				3
#define RAW_TYPE_FLOAT_STEREO_LEFT			4
#define RAW_TYPE_FLOAT_STEREO_RIGHT			5
#define WAV_TYPE_MONO					6
#define WAV_TYPE_STEREO_LEFT				7
#define WAV_TYPE_STEREO_RIGHT				8
#define INPUT_TYPE_AUTOMATIC				0x0defaced

struct channel;

typedef struct {
  int type;
  int (*feed_buffer_short_mono)(void *, short *, int, int);
  int (*feed_buffer_float_mono)(void *, float *, int, int);
  int (*feed_buffer_short_stereo)(void *, short *, int, int);
  int (*feed_buffer_float_stereo)(void *, float *, int, int);
  int (*feed_buffer_short_n_channels)(int, void *, short *, int, int);
  int (*feed_buffer_float_n_channels)(int, void *, float *, int, int);
  int (*first_point)(void *);
  int (*last_point)(void *);
  void (*reset)(void *);
  struct channel *subchannel;
  void (*open_gui)(void *, int);
  void (*close_gui)(void *);
  int gui_on:1;
  int bypass:1;
  void *label;
} channel_common;

/* all various channel types will "derivate" from this basic one */
typedef struct channel {
  channel_common c;
} channel;

char *channel_type_to_char(int);

channel *new_channel(int, channel *);

void delete_channel(channel *);
void delete_channel_and_sub(channel *);

channel *channel_getsub(channel *, int);
channel *channel_remove(channel *, channel *);
channel *channel_insert(channel *, channel *, int);

#endif /* _CHANNEL_H_ */
