/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "effects_gui.h"

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "track.h"
#include "error.h"
#include "mixer.h"
#include "main_gui.h"
#include "channel.h"
#include "yes_no_gui.h"
#include "version.h"

static int effects_gui_on = 0;

static GList *tracks = 0;
static GtkWidget *combo;
static GtkWidget *win;
static GtkListStore *effects_list;
static GtkWidget *tree_view;

static track *current_track = 0;

static void try_free(char *c)
{
  /* "Mixer track" must not be freed */
  if (c[0] == 'M') return;
  free(c);
}

static void free_list(GList *l)
{
  g_list_foreach(l, (GFunc)try_free, 0);
  g_list_free(l);
}

static int setup_track_list(void)
{
  char *l;
  GList *new;
  int i;

  if (tracks) {
    free_list(tracks);
    tracks = 0;
  }

  tracks = g_list_append(tracks, "Mixer track");
  if (!tracks) {
    oom();
    display_error();
    return -1;
  }

  for (i=0; i<main_mixer->nb_tracks; i++) {
    /* error used as buffer */
    reset_error();
    add_error_char("Track # ");
    add_error_int(i+1);

    l = strdup(get_error());
    if (!l) {
      oom();
      display_error();
      free_list(tracks);
      tracks = 0;
      return -1;
    }

    new = g_list_append(tracks, l);
    if (!new) {
      oom();
      display_error();
      free_list(tracks);
      tracks = 0;
      return -1;
    }
    tracks = new;
  }

  current_track = 0;

  gtk_combo_set_popdown_strings (GTK_COMBO (combo), tracks);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), "");
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), tracks->data);

  return 0;
}

static void toggle_event(GtkCellRendererToggle *r, gchar *t, int column)
{
  GtkTreeIter iter;
  int m = atoi(t);
  gboolean tog;
  channel *c;

  gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(effects_list), &iter, t);

  if (gtk_cell_renderer_toggle_get_active(r))
    tog = FALSE;
  else
    tog = TRUE;

  c = current_track->channel;

  while (m && c) {
    m--;
    c = c->c.subchannel;
  }

  if (!c)
    return;

  if (column == 0) {
    if (tog == TRUE)
      c->c.open_gui(c, current_track->track_number);
    else
      c->c.close_gui(c);
  } else if (column == 1) {
    if (c->c.type == CHANNEL_TYPE_POS) {
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION == 0
      reset_error();
      add_error_char("You cannot bypass the 'Position' effect.\n");
      display_error();
#else
      gdk_display_beep(gdk_display_get_default());
#endif
      return;
    }
    c->c.bypass = tog == TRUE ? 1 : 0;
  }

  modify = 1;

  gtk_list_store_set(effects_list, &iter, column, tog, -1);
}

static void up_event(void)
{
  GtkTreeIter iter;
  int effect_number;
  channel *n;
  channel *new;
  int i;

  if (!gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), 0, &iter))
    return;

  gtk_tree_model_get(GTK_TREE_MODEL(effects_list), &iter, 3, &effect_number, -1);

  if (effect_number == 0)
    return;

  n = channel_getsub(current_track->channel, effect_number);

  if (!n) {
    fprintf(stderr, "yoyo\n");
    return;
  }

  if (n->c.type == CHANNEL_TYPE_POS) {
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION == 0
    reset_error();
    add_error_char("You cannot move the 'Position' effect.\n");
    display_error();
#else
    gdk_display_beep(gdk_display_get_default());
#endif
    return;
  }

  new = channel_remove(current_track->channel, n);

  if (!new) {
    fprintf(stderr, "yoyo 2\n");
    return;
  }

  new = channel_insert(new, n, effect_number - 1);

  if (!new) {
    fprintf(stderr, "yoyo 4 %d\n", effect_number);
    return;
  }

  current_track->channel = new;

  modify = 1;

  effects_gui_force_redraw();

  /* try to reselect the line */
  if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(effects_list), &iter))
    return;
  for (i = 0; i < effect_number-1; i++)
    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(effects_list), &iter))
      return;

  gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), &iter);
}

static void down_event(void)
{
  GtkTreeIter iter;
  int effect_number;
  channel *n;
  channel *new;
  int i;

  if (!gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), 0, &iter))
    return;

  gtk_tree_model_get(GTK_TREE_MODEL(effects_list), &iter, 3, &effect_number, -1);

  n = channel_getsub(current_track->channel, effect_number);

  if (!n) {
    fprintf(stderr, "yoyo\n");
    return;
  }

  if (n->c.type == CHANNEL_TYPE_POS)
    return;

  if (n->c.subchannel->c.type == CHANNEL_TYPE_MIXER)
    return;

  if (n->c.subchannel->c.type == CHANNEL_TYPE_POS) {
#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION == 0
    reset_error();
    add_error_char("You cannot move below the 'Position' effect.\n");
    display_error();
#else
    gdk_display_beep(gdk_display_get_default());
#endif
    return;
  }

  new = channel_remove(current_track->channel, n);

  if (!new) {
    fprintf(stderr, "yoyo 2\n");
    return;
  }

  new = channel_insert(new, n, effect_number + 1);

  if (!new) {
    fprintf(stderr, "yoyo 3\n");
    return;
  }

  current_track->channel = new;

  modify = 1;

  effects_gui_force_redraw();

  /* try to reselect the line */
  if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(effects_list), &iter))
    return;
  for (i = 0; i < effect_number+1; i++)
    if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(effects_list), &iter))
      return;

  gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), &iter);
}

static GtkWidget *effect_combo;

static void new_chosen_event(void *unused, int what)
{
  channel *new;
  char *c = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(effect_combo)->entry));
  int chosen_effect = FIRST_EFFECTABLE_CHANNEL;

  if (what != GTK_RESPONSE_ACCEPT)
    return;

  while (strcmp(c, channel_type_to_char(chosen_effect)))
    chosen_effect++;

  new = new_channel(chosen_effect, current_track->channel);

  if (!new)
    return;

  current_track->channel = new;

  modify = 1;

  effects_gui_force_redraw();
}

static void new_event(int effect_type)
{
  GtkWidget *new_win;
  GtkWidget *vbox;
  GtkWidget *label;
  GList *list = 0;
  char *t;
  int effect;

  /* feeding list */
  effect = FIRST_EFFECTABLE_CHANNEL;
  t = channel_type_to_char(effect);
  while (strcmp(t, UNKNOWN_CHANNEL_TYPE)) {
    GList *new = g_list_append(list, t);
    if (!new) {
      if (list) g_list_free(list);
      oom();
      display_error();
      return;
    }
    list = new;
    effect++;
    t = channel_type_to_char(effect);
  }

  /* error used as buffer */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  add_error_char(" - new effect");

  new_win = gtk_dialog_new_with_buttons(get_error(), GTK_WINDOW(win), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
        GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
        GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
        0);

  vbox = GTK_DIALOG(new_win)->vbox;
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

  /* a little explaination */
  label = gtk_label_new("Choose the effect, then press OK\n");
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

  /* then the effect list */
  effect_combo = gtk_combo_new();
  gtk_box_pack_start(GTK_BOX(vbox), effect_combo, TRUE, TRUE, 0);
  gtk_combo_set_popdown_strings(GTK_COMBO(effect_combo), list);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(effect_combo)->entry), FALSE);

  /* connect signals */
  g_signal_connect(new_win, "response", G_CALLBACK(new_chosen_event), 0);
  g_signal_connect(new_win, "response", G_CALLBACK(gtk_widget_destroy), 0);

  gtk_widget_show_all(new_win);
}

static channel *to_be_deleted;

static void delete_anyway(void)
{
  channel *new;

  new = channel_remove(current_track->channel, to_be_deleted);

  if (!new) {
    fprintf(stderr, "yoyo 2\n");
    return;
  }

  delete_channel(to_be_deleted);

  current_track->channel = new;

  modify = 1;

  effects_gui_force_redraw();
}

static void delete_event(void)
{
  GtkTreeIter iter;
  int effect_number;
  channel *n;

  if (!gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), 0, &iter))
    return;

  gtk_tree_model_get(GTK_TREE_MODEL(effects_list), &iter, 3, &effect_number, -1);

  n = channel_getsub(current_track->channel, effect_number);

  if (!n)
    return;

  if (n->c.type == CHANNEL_TYPE_POS) {
    reset_error();
    add_error_char("'Position' cannot be deleted.\n");
    display_error();
    return;
  }

  to_be_deleted = n;

  yes_no(win, "Are you sure you want to delete this effect?\n(no undo possible)\n", delete_anyway, 0);
}

static void combo_entry_event(void)
{
  track *t;
  int track_number;
  channel *c;
  GtkTreeIter iter;
  const char *val;
  int i;

  val = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));

  /* it is very possible to get a null string here */
  if (!val || val[0] == 0)
    return;

  /* what track is it? */
  if (!strcmp(val, "Mixer track"))
    t = &main_mixer->mixer_track;
  else {
    if (sscanf(val, "Track # %d", &track_number) != 1) {
      reset_error();
      add_error_char("Contact the authors with the following message:\n");
      add_error_char(__FILE__);
      add_error_char(":");
      add_error_int(__LINE__);
      add_error_char(":");
      add_error_char(__FUNCTION__);
      add_error_char(": \"");
      add_error_char((char *)val);
      add_error_char("\"\nAnd quit the program, therre is something strange in there (save your data first)\n");
      display_error();
      return;
    }
    t = mixer_get_track(main_mixer, track_number-1);
  }

  if (t == current_track)
    return;

  gtk_list_store_clear(effects_list);

  c = t->channel;
  i = 0;

  while (c->c.type != CHANNEL_TYPE_INPUT && c->c.type != CHANNEL_TYPE_MIXER) {
    gtk_list_store_append(effects_list, &iter);
    gtk_list_store_set(effects_list, &iter, 0, c->c.gui_on?TRUE:FALSE, 1, c->c.bypass?TRUE:FALSE, 2, channel_type_to_char(c->c.type), 3, i, -1);
    c = c->c.subchannel;
    i++;
  }

  current_track = t;
}

static void destroy_event(void)
{
  free_list(tracks);
  tracks = 0;
  gtk_list_store_clear(effects_list);
  g_object_unref(effects_list);
  current_track = 0;
  effects_gui_on = 0;
}

static void effects_gui_create(void)
{
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *scrolled_win;
  GtkCellRenderer *render1, *render2, *render3;
  GtkTreeViewColumn *col1, *col2, *col3;
  GtkWidget *delete;
  GtkWidget *new;
  GtkWidget *up;
  GtkWidget *down;
  GtkWidget *close;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  {
    /* let's force its minimal size to be 1x1, I don't like to let the user with a too big window if she wants to resize it smaller */
    GdkGeometry g;
    g.min_width = g.min_height = 1;
    gtk_window_set_geometry_hints(GTK_WINDOW(win), win, &g, GDK_HINT_MIN_SIZE);
  }

  gtk_widget_set_size_request(win, -1, 250);

  /* error used as buffer */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  add_error_char(" - effects");
  gtk_window_set_title(GTK_WINDOW(win), get_error());

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  /* in top, a combo that lets the user choose the track she wants to edit */
  combo = gtk_combo_new ();
  gtk_box_pack_start(GTK_BOX(vbox), combo, FALSE, FALSE, 0);

  setup_track_list();

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), FALSE);
  GTK_WIDGET_SET_FLAGS(GTK_COMBO(combo)->entry, GTK_CAN_FOCUS);

  /* then, the list of effects for the current track */
  effects_list = gtk_list_store_new(4, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INT);
  scrolled_win = gtk_scrolled_window_new(0, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
  tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(effects_list));
  gtk_container_add(GTK_CONTAINER(scrolled_win), tree_view);
  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)), GTK_SELECTION_SINGLE);
  render1 = gtk_cell_renderer_toggle_new();
  render2 = gtk_cell_renderer_toggle_new();
  render3 = gtk_cell_renderer_text_new();
  col1 = gtk_tree_view_column_new_with_attributes("View", render1, "active", 0, 0);
  col2 = gtk_tree_view_column_new_with_attributes("Bypass", render2, "active", 1, 0);
  col3 = gtk_tree_view_column_new_with_attributes("Effect name", render3, "text", 2, 0);
  //gtk_tree_view_column_set_resizable(col1, TRUE);
  //gtk_tree_view_column_set_resizable(col2, TRUE);
  gtk_tree_view_column_set_resizable(col1, FALSE);
  gtk_tree_view_column_set_resizable(col2, FALSE);
  gtk_tree_view_column_set_resizable(col3, FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), col1);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), col2);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), col3);

  /* then buttons */
  hbox = gtk_hbox_new(TRUE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  delete = gtk_button_new_with_label("Delete");
  new = gtk_button_new_with_label("New");
  up = gtk_button_new_with_label("Up");
  down = gtk_button_new_with_label("Down");
  gtk_box_pack_start(GTK_BOX(hbox), up, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), down, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), new, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), delete, TRUE, TRUE, 0);

  /* then close button */
  close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(vbox), close, FALSE, FALSE, 0);

  /* connect signals */
  g_signal_connect(render1, "toggled", G_CALLBACK(toggle_event), 0);
  g_signal_connect(render2, "toggled", G_CALLBACK(toggle_event), (void *)1);
  g_signal_connect(GTK_COMBO(combo)->entry, "changed", combo_entry_event, 0);

  g_signal_connect(up, "clicked", G_CALLBACK(up_event), 0);
  g_signal_connect(down, "clicked", G_CALLBACK(down_event), 0);
  g_signal_connect(new, "clicked", G_CALLBACK(new_event), 0);
  g_signal_connect(delete, "clicked", G_CALLBACK(delete_event), 0);

  g_signal_connect_swapped(close, "clicked", G_CALLBACK(gtk_widget_destroy), win);

  g_signal_connect(win, "destroy", G_CALLBACK(destroy_event), 0);

  gtk_widget_show_all(win);

  /* this MUST come last */
  effects_gui_on = 1;
}

void effects_gui_show_track(track *t)
{
  char tn[64];

  if (current_track == t)
    return;

  if (!effects_gui_on) {
    effects_gui_create();
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), "");
  }

  if (t == &main_mixer->mixer_track) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), "Mixer track");
    return;
  }

  sprintf(tn, "Track # %d", t->track_number+1);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), tn);
}

void effects_gui_new_track(track *t)
{
  if (!effects_gui_on)
    return;

  setup_track_list();
}

void effects_gui_delete_track(track *t)
{
  if (!effects_gui_on)
    return;

  setup_track_list();
}

void effects_gui(void)
{
  if (effects_gui_on)
    return;

  effects_gui_show_track(&main_mixer->mixer_track);
}

void effects_gui_force_redraw(void)
{
  char *c;
  char t[64];

  if (!effects_gui_on)
    return;

  c = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));

  if (!c || c[0] == 0)
    c = "Mixer track";

  if (strlen(c) > 63) {
    reset_error();
    add_error_char("Display refresh error, contact the authors.\n");
    add_error_char("This is not a big problem, but still a problem.\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": this information must be sent to the authors, thank you.\n");
    display_error();
    return;
  }

  strcpy(t, c);

  current_track = 0;

  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), "");
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), t);
}
