/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "error.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gui.h"
#include "error_gui.h"

static error glo;

void open_error(void)
{
  memset(&glo, 0, sizeof(error));

  glo.data = malloc(4096);
  if (!glo.data) {
    fprintf(stderr, "no memory to init error handling, adios!\n");
    exit(1);
  }
  glo.maxsize = 4096;
  glo.size = 1;
  glo.data[0] = 0;
}

void close_error(void)
{
  free(glo.data);
}

void reset_error(void)
{
  glo.size = 1;
  glo.data[0] = 0;

  glo.internal_out_of_memory = 0;
}

void add_error_char(char *s)
{
  char *new;
  int l = strlen(s);

  if (glo.internal_out_of_memory) {
    fprintf(stderr, "%s\n", s);
    return;
  }

  if (glo.size + l > glo.maxsize) {
    while (glo.maxsize < glo.size + l)
      glo.maxsize += 4096;
    new = realloc(glo.data, glo.maxsize);
    if (!new) {
      fprintf(stderr, "There is an error in the error handling!\nContact the authors!\n");
      fprintf(stderr, "Here is the current error:\n");
      fprintf(stderr, "%s%s\n", glo.data, s);
      glo.internal_out_of_memory = 1;
      return;
    }
    glo.data = new;
  }

  strcpy(&glo.data[glo.size-1], s);

  glo.size += l;
}

void add_error_int(int a)
{
  static char c[64];

  sprintf(c, "%d", a);

  add_error_char(c);
}

void oom(void)
{
  reset_error();
  add_error_char("Sorry, you are out of memory,\n");
  add_error_char("try to save your data and quit the program.\n");
}

char *get_error(void)
{
  if (glo.internal_out_of_memory)
    return "Almost fatal error,\nyou are out of memory,\ntry to save your data,\ncontact the authors,\nthis is a big problem here...\n";

  return glo.data;
}

void display_error(void)
{
  if (!gui_ok) {
    fprintf(stderr, "%s\n", glo.data);
    return;
  }

  display_error_gui(glo.data);
}

void display_error_modal(void)
{
  if (!gui_ok) {
    fprintf(stderr, "%s\n", glo.data);
    return;
  }

  display_error_gui_modal(glo.data);
}
