/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "file_gui.h"

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "error.h"
#include "channel.h"

static int ok;

static GtkWidget *file_selector;

static char last_file_dir[FILENAME_MAX+1] = { 0 };
static char file_name[FILENAME_MAX+1];

void file_event(void)
{
  char *name = (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
  int len;
  int i;

  last_file_dir[0] = 0;
  file_name[0] = 0;

  if ((len=strlen(name)) <= FILENAME_MAX) {
    strcpy(file_name, name);
    strcpy(last_file_dir, name);
    for (i=len-1; i>=0; i--)
      if (last_file_dir[i] == '/') {
        last_file_dir[i+1] = 0;
        break;
      }
  }

  ok = 1;
}

static void destroy_event(void)
{
  gtk_main_quit();
}

/* this function is not recursivable, you should be sure fn won't call it again, otherwise, big troubles */
void new_file_selector(char *lab, void (*fn)(char *), void *parent)
{
  GtkWidget *win;
  GtkWidget *label = gtk_label_new(lab);
  GtkWidget *sep = gtk_hseparator_new();

  win = gtk_file_selection_new(lab);

  if (last_file_dir[0])
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(win), last_file_dir);

  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(win)->main_vbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(win)->main_vbox), sep, FALSE, FALSE, 10);
  gtk_box_reorder_child(GTK_BOX(GTK_FILE_SELECTION(win)->main_vbox), label, 0);
  gtk_box_reorder_child(GTK_BOX(GTK_FILE_SELECTION(win)->main_vbox), sep, 1);

  file_selector = win;

  g_signal_connect(GTK_FILE_SELECTION(win)->ok_button, "clicked", G_CALLBACK(file_event), 0);

  g_signal_connect_swapped(GTK_FILE_SELECTION(win)->ok_button, "clicked", G_CALLBACK(gtk_widget_destroy), win);
  g_signal_connect_swapped(GTK_FILE_SELECTION(win)->cancel_button, "clicked", G_CALLBACK(gtk_widget_destroy), win);

  g_signal_connect(win, "destroy", G_CALLBACK(destroy_event), 0);

  /* I don't like modal stuff, but it's more simpler here (I tried to do it without it, it becomes very very complex,
   * I don't understand all what happens, the program may crash and I don't want that)
   */
  gtk_window_set_modal(GTK_WINDOW(win), TRUE);

  gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(parent));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);

  gtk_widget_show_all(win);

  ok = 0;

  gtk_main();

  if (ok)
    fn(file_name);
}

static int ok=0;
static char choice[64];

static void ok_event(GtkWidget *combo)
{
  /* no buffer overflow, see below */
  strcpy(choice, (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry)));
  ok = 1;
}

static GList *_g_list_append(GList *l, char *n)
{
  /* we use a static buffer of 64 bytes to store the choice of the user,
   * so we do a little check here in case the code would have changed
   * in future releases. (yeah ! I master english grammar !)
   */
  if (strlen(n) > 63) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": you should not read this, contact the authors.\n");
    display_error();
    return 0;
  }

  return g_list_append(l, n);
}

int ask_user_file_type(char *file)
{
  extern GtkWidget *main_win;
  GtkWidget *win;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *combo;
  GtkWidget *ok, *cancel;
  GList *list, *new;
  char *c;

  ok = 0;

  /* building list */
  list = 0;
  new = _g_list_append(list, "signed 16 bits little endian mono"); if (!new) goto error; list = new;
  new = _g_list_append(list, "signed 16 bits little endian stereo"); if (!new) goto error; list = new;
  new = _g_list_append(list, "float 32 bits mono"); if (!new) goto error; list = new;
  new = _g_list_append(list, "float 32 bits stereo"); if (!new) goto error; list = new;

  /* gui stuff */
  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(main_win));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  gtk_container_set_border_width(GTK_CONTAINER(win), 5);
  /* modal is bad, but hey... */
  gtk_window_set_modal(GTK_WINDOW(win), TRUE);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(file), FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new("The file type is not recognize or not handled, load it as:"), FALSE, FALSE, 0);

  combo = gtk_combo_new();
  gtk_box_pack_start(GTK_BOX(vbox), combo, FALSE, FALSE, 0);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), FALSE);
  gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);

  hbox = gtk_hbox_new(TRUE, 10);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  ok = gtk_button_new_with_mnemonic("_OK");
  cancel = gtk_button_new_with_mnemonic("_Cancel");
  gtk_box_pack_start(GTK_BOX(hbox), ok, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), cancel, TRUE, TRUE, 0);

  gtk_widget_show_all(win);

  /* callbacks */
  g_signal_connect(win, "destroy", G_CALLBACK(gtk_main_quit), 0);
  g_signal_connect_swapped(ok, "clicked", G_CALLBACK(ok_event), combo);
  g_signal_connect_swapped(ok, "clicked", G_CALLBACK(gtk_widget_destroy), win);
  g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(gtk_widget_destroy), win);

  /* let's wait user's answer */
  gtk_main();

  if (!ok)
    return -1;

  c = choice;

  if (!strcmp(c, "signed 16 bits little endian mono"))
    return RAW_TYPE_16B_LITTLE_ENDIAN_MONO;

  if (!strcmp(c, "signed 16 bits little endian stereo"))
    return RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_LEFT;

  if (!strcmp(c, "float 32 bits mono"))
    return RAW_TYPE_FLOAT_MONO;

  if (!strcmp(c, "float 32 bits stereo"))
    return RAW_TYPE_FLOAT_STEREO_LEFT;

  /* we should not come here, but hey... you know... */
  return -1;

error:
  if (list)
    g_list_free(list);

  return -1;
}
