/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "input.h"

#include <stdlib.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "endian.h"
#include "raw.h"
#include "wav.h"
#include "error.h"
#include "memmap.h"

int mono_short_little_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 2);

  memcpy(buf, d, nb_samples*2);
#if 0
  for (i=0; i<nb_samples; i++)
    *buf++ = *d++;
#endif

  return 1;
}

int mono_short_little_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
  }

  return 1;
}

int mono_short_little_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      *buf++ = (short)v;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
  }

  return 1;
}

int mono_short_little_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      *buf++ = (float)((short)v);
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
  }

  return 1;
}

int stereo_left_short_little_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_short_little_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_short_little_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (short)v;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_short_little_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p += 2;
      *buf++ = (float)((short)v);
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_short_little_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4 + 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4 + 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_short_little_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4 + 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4 + 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_short_little_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4 + 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p+=2;
      *buf++ = (short)v;
      *buf++ = (short)v;
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4 + 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_short_little_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  short *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    unsigned short v;
    unsigned char *p = (unsigned char *)this->data + start * 4 + 2;
    for (i=0; i<nb_samples; i++) {
      v = *p++;
      v <<= 8;
      v += *p++;
      p += 2;
      *buf++ = (float)((short)v);
      *buf++ = (float)((short)v);
    }
    return 1;
  }

  d = (short *)((char *)this->data + start * 4 + 2);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

/********************************************/

int mono_float_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++)
    *buf++ = *d++;

  return 1;
}

int mono_float_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
  }

  return 1;
}

int mono_float_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
  }

  return 1;
}

int mono_float_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
  }

  return 1;
}

int stereo_left_float_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_float_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_float_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_left_float_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_float_feed_buffer_short_mono(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8 + 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_float_feed_buffer_float_mono(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8 + 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_float_feed_buffer_short_stereo(input *this, short *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8 + 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int stereo_right_float_feed_buffer_float_stereo(input *this, float *buf, int nb_samples, int start)
{
  int i;
  float *d;

  /* out of buffer, don't feed anything (not accurate, but the caller must know what she does) */
  if (start < 0 || nb_samples+start > this->size)
    return 0;

  if (big_endian == 1) {
    /* what to do ? */
    return 0;
  }

  d = (float *)((char *)this->data + start * 8 + 4);

  for (i=0; i<nb_samples; i++) {
    *buf++ = *d;
    *buf++ = *d++;
    d++;
  }

  return 1;
}

int input_first_point(input *this)
{
  return 0;
}

int input_last_point(input *this)
{
  return this->size - 1;
}

void input_reset(input *this)
{
  /* do nothing */
}

/* input files are shared, so we mmap only once and we calcute view only once (it eats up memory) */
typedef struct {
  input *i;
  dev_t device;
  ino_t inode;
  int count;
} input_list;

static input_list *list = 0;
static int size = 0;
static int maxsize = 0;

input *new_input(char *f, int type)
{
  input *new;
  int i;
  struct stat buf;

  /* we don't compare filenames, we compare devices and inodes */
  if (stat(f, &buf) == -1) {
    int serr = errno;
    reset_error();
    add_error_char("New track\n");
    add_error_char(f);
    add_error_char(": ");
    add_error_char(strerror(serr));
    display_error();
    return 0;
  }

  /* let's run through the list, to see if it is already there or not */
  for (i=0; i<size; i++)
    if (buf.st_dev == list[i].device && buf.st_ino == list[i].inode && type == list[i].i->input_type) {
      list[i].count++;
      return list[i].i;
    }

  /* it's not, let's put it */
  if (size == maxsize) {
    input_list *n = realloc(list, (maxsize + 32) * sizeof(input_list));
    if (!n) {
      oom();
      display_error();
      return 0;
    }
    list = n;
    maxsize += 32;
  }

  switch (type) {
    case RAW_TYPE_16B_LITTLE_ENDIAN_MONO:
    case RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_LEFT:
    case RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_RIGHT:
    case RAW_TYPE_FLOAT_MONO:
    case RAW_TYPE_FLOAT_STEREO_LEFT:
    case RAW_TYPE_FLOAT_STEREO_RIGHT:
      new = new_raw(f, 44100, type);
      if (!new) return 0;
      list[size].i = new;
      list[size].device = buf.st_dev;
      list[size].inode = buf.st_ino;
      list[size].count = 1;
      size++;
      return new;

    case WAV_TYPE_MONO:
    case WAV_TYPE_STEREO_LEFT:
    case WAV_TYPE_STEREO_RIGHT:
      new = new_wav(f, type);
      if (!new) return 0;
      list[size].i = new;
      list[size].device = buf.st_dev;
      list[size].inode = buf.st_ino;
      list[size].count = 1;
      size++;
      return new;
  }

  reset_error();
  add_error_char("The type ");
  add_error_int(type);
  add_error_char(" is not supported.\n");
  add_error_char("Songs cannot load ");
  add_error_char(f);
  add_error_char("\n");
  display_error();

  return 0;
}

void delete_input(input *r)
{
  int i;

  /* it has to be in the list */
  for (i=0; i<size; i++)
    if (list[i].i == r)
      break;

  if (i == size) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": you should not read this, contact the authors.\n");
    display_error();
    return;
  }

  list[i].count--;

  if (list[i].count)
    return;

  switch (r->input_type) {
    case RAW_TYPE_16B_LITTLE_ENDIAN_MONO:
    case RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_LEFT:
    case RAW_TYPE_16B_LITTLE_ENDIAN_STEREO_RIGHT:
    case RAW_TYPE_FLOAT_MONO:
    case RAW_TYPE_FLOAT_STEREO_LEFT:
    case RAW_TYPE_FLOAT_STEREO_RIGHT:
      memunmap(r->data, r->file_size);
      break;
    case WAV_TYPE_MONO:
    case WAV_TYPE_STEREO_LEFT:
    case WAV_TYPE_STEREO_RIGHT:
      memunmap((char *)r->data-44, r->file_size);
      break;
  }

  if (r->view_min) {
    free(r->view_min);
    free(r->view_max);
    r->view_min = r->view_max = 0;
  }

  free(r->file);
  free(r);

  /* we have to remove it from the list */
  for (; i<size-1; i++)
    list[i] = list[i+1];

  size--;

  /* make some room */
  if (maxsize-size > 32) {
    input_list *new = realloc(list, (maxsize-32) * sizeof(input_list));
    /* if it does not work, no problem */
    if (!new)
      return;
    maxsize -= 32;
    list = new;
  }
}

void cleanup_input(void)
{
  if (size) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": you should not read this, contact the authors.\n");
    display_error();
  }

  if (!maxsize)
    return;

  free(list);
  list = 0;
  maxsize = 0;
}
