/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "main_gui.h"

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "version.h"
#include "error.h"
#include "track.h"
#include "file_gui.h"
#include "audio.h"
#include "record_gui.h"
#include "mixer.h"
#include "file.h"
#include "yes_no_gui.h"
#include "effects_gui.h"
#include "vol.h"
#include "help_gui.h"
#include "soundcard.h"
#include "view_gui.h"
#include "channel.h"
#include "pos.h"
#include "mark_gui.h"
#include "track_gui.h"
#include "toolbox_gui.h"
#include "mix_gui.h"
#include "dump_gui.h"
#include "setup_gui.h"

int modify = 0;

GtkWidget *main_win;
static GtkWidget *track_list_left;
GtkWidget *track_list_right;
GtkWidget *marker_track_list_right;
static GtkWidget *main_combo_play_start;
static GtkWidget *main_combo_play_stop;
GtkWidget *main_combo_marker_list;
static GtkWidget *left_box;
GtkWidget *scrollbar_right_horiz;
GtkWidget *scrollbar_right_vert;

#if 0
static GtkWidget *realtime_vol;
static GtkWidget *realtime_pan;
static GtkWidget *realtime_volenv;
static GtkWidget *realtime_panenv;
static GtkWidget *realtime_eq;
static GtkWidget *realtime_compressor;
static GtkWidget *precalc_reverb;
static GtkWidget *precalc_reverb_stereo;
#endif

static GtkWidget *panned;
static GtkWidget *panned_left;
static GtkWidget *panned_right;

GdkCursor *track_move;
GdkCursor *track_left_arrow;
GdkCursor *track_right_arrow;

GdkGC *red_gc;

static char current_file[FILENAME_MAX+1];

static void do_save(char *c)
{
  if (strlen(c) > FILENAME_MAX) {
    reset_error();
    add_error_char(c);
    add_error_char(": filename too long, I can't handle it (");
    add_error_int(FILENAME_MAX);
    add_error_char(" is the max length).\n");
    display_error();
    return;
  }

  if (save(c) == -1)
    return;

  strcpy(current_file, c);
}

static void do_open(char *c)
{
  if (strlen(c) > FILENAME_MAX) {
    reset_error();
    add_error_char(c);
    add_error_char(": filename too long, I can't handle it (");
    add_error_int(FILENAME_MAX);
    add_error_char(" is the max length).\n");
    display_error();
    return;
  }

  if (load(c) == -1)
    return;

  strcpy(current_file, c);
}

static void new_anyway(void)
{
  int i;
  int nb = main_mixer->nb_tracks;
  channel *new;

  for (i=0; i<nb; i++)
    delete_track(mixer_get_track(main_mixer, 0));

  nb = mark_list_size;

  for (i=0; i<nb; i++)
    delete_mark(get_nth_mark(0));

  new = (channel *)new_vol(&main_mixer->_raw_mix);
  if (new) {
    delete_channel_and_sub(main_mixer->mixer_track.channel);
    main_mixer->mixer_track.channel = new;
    effects_gui_force_redraw();
  }

  current_file[0] = 0;

  modify = 0;
}

static void new_event(void)
{
  if (modify) {
    /* error used as buffer */
    reset_error();
    add_error_char("Your current project has not been saved.\n");
    add_error_char("Are you sure you want to create a new one?\n");
    add_error_char("(if you don't save your current project,\nyou will lose some of your work)\n");
    yes_no(main_win, get_error(), new_anyway, 0);
    return;
  }

  new_anyway();
}

static void open_callback(char *name)
{
  do_open(name);
}

static void open_anyway(void)
{
  new_file_selector("Open a project:\n", open_callback, main_win);
}

static void open_event(void)
{
  if (modify) {
    /* error used as buffer */
    reset_error();
    add_error_char("Your current project has not been saved.\n");
    add_error_char("Are you sure you want to open another one?\n");
    add_error_char("(if you don't save your current project,\nyou will lose some of your work)\n");
    yes_no(main_win, get_error(), open_anyway, 0);
    return;
  }

  open_anyway();
}

static char saveas_filename[FILENAME_MAX+1];

static void saveas_anyway(void)
{
  do_save(saveas_filename);
}

static void saveas_callback(char *n)
{
  struct stat stat_buf;

  if (strlen(n) > FILENAME_MAX) {
    reset_error();
    add_error_char(n);
    add_error_char(": filename too long, I can't handle it (");
    add_error_int(FILENAME_MAX);
    add_error_char(" is the max length).\n");
    display_error();
    return;
  }

  /* if file already exists, ask confirmation */
  if (stat(n, &stat_buf) != -1) {
    strcpy(saveas_filename, n);
    /* error used as buffer */
    reset_error();
    add_error_char("The file ");
    add_error_char(n);
    add_error_char(" already exists.\n");
    add_error_char("Do you want to overwrite it?\n");
    yes_no(main_win, get_error(), saveas_anyway, 0);
    return;
  }

  do_save(n);
}

static void saveas_event(void)
{
  new_file_selector("Save project as:\n", saveas_callback, main_win);
}

static void save_event(void)
{
  if (current_file[0] == 0)
    return saveas_event();

  do_save(current_file);
}

static void main_quit_anyway(void)
{
  int nb = main_mixer->nb_tracks;
  int i;

  for (i=0; i<nb; i++)
    delete_track(mixer_get_track(main_mixer, 0));

  nb = mark_list_size;

  for (i=0; i<nb; i++)
    delete_mark(get_nth_mark(0));

  delete_channel_and_sub(main_mixer->mixer_track.channel);

  delete_mixer(main_mixer);

  gtk_widget_destroy(GTK_WIDGET(main_win));

  gtk_main_quit();

  current_file[0] = 0;
}

static void main_quit_event(void)
{
  if (modify) {
    /* error used as buffer */
    reset_error();
    add_error_char("Your current project has not been saved.\n");
    add_error_char("Are you sure you want to quit?\n");
    add_error_char("(if you don't save your current project,\nyou will lose some of your work)\n");
    yes_no(main_win, get_error(), main_quit_anyway, 0);
    return;
  }

  main_quit_anyway();
}

static void do_newtrack(char *c)
{
  new_track(c, INPUT_TYPE_AUTOMATIC);
}

static void newtrack_event(void)
{
  new_file_selector("Add a new track, choose the file:\n", do_newtrack, main_win);
}

static void play_event(void)
{
  mark *m;
  char *c;
  int r;

  r = open_audio_play(PLAY_SOUND);

  if (r == -1 || r == -2)
    return;

  /* calculating first point */
  c = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry));
  m = get_mark(c);
  if (!m)
    main_mixer->next_sample = first_audio_sample();
  else
    main_mixer->next_sample = m->sample;

  /* calculating last point */
  main_mixer->has_last_sample = 1;
  c = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry));
  m = get_mark(c);
  if (!m) {
    if (!strcmp(c, "Infinity"))
      main_mixer->has_last_sample = 0;
    else
      main_mixer->last_sample = last_audio_sample();
  } else
    main_mixer->last_sample = m->sample;

  play_mark_on(main_mixer->next_sample);
}

static void stop_event(void)
{
  close_audio();
}

static void record_event(void)
{
  record_gui();
}

static void effects_event(void)
{
  effects_gui();
}

static int first_view_point(track *t)
{
  channel *c = t->channel;

  while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;

  return ((pos *)c)->global_start;
}

static int last_view_point(track *t)
{
  channel *p = 0 /* for no gcc warnings */;
  channel *i;

  i = t->channel;

  while (i->c.type != CHANNEL_TYPE_INPUT) {
    if (i->c.type == CHANNEL_TYPE_POS) p = i;
    i = i->c.subchannel;
  }

  return ((pos *)p)->global_start + i->c.last_point(i);
}

void main_gui_update_track_view(void)
{
  track *t;
  int min_point;
  int max_point;
  int i;

  if (!main_mixer->nb_tracks) {
    main_mixer->minimum_sample = -10*4096;
    main_mixer->maximum_sample = 0;
    min_point = -10*4096;
    max_point = 10*4096;
    goto after;
  }

  t = main_mixer->tracks[0];

  min_point = first_view_point(t) -10*4096;
  max_point = last_view_point(t) +10*4096;

  for (i=1; i<main_mixer->nb_tracks; i++) {
    int new_min;
    int new_max;

    t = main_mixer->tracks[i];

    new_min = first_view_point(t) -10*4096;
    new_max = last_view_point(t) +10*4096;

    if (new_min < min_point)
      min_point = new_min;

    if (new_max > max_point)
      max_point = new_max;
  }

  /* we reposition what has changed of place */
  for (i=0; i<main_mixer->nb_tracks; i++) {
    int newx;
    track *t = main_mixer->tracks[i];
    pos *p = (pos *)t->channel;
    while (p->c.type != CHANNEL_TYPE_POS) p = (pos *)p->c.subchannel;
    newx = (p->global_start-min_point+4095)/4096;
    if (newx != t->drawing_x) {
      t->drawing_x = newx;
      gtk_layout_move(GTK_LAYOUT(track_list_right), t->drawing_area, t->drawing_x, t->drawing_y);
    }
  }

after:
  main_mixer->minimum_sample = min_point;
  main_mixer->maximum_sample = max_point-10*4096;

  min_point += 4095;
  min_point /= 4096;
  max_point += 4095;
  max_point /= 4096;

  main_mixer->gui_right_size = max_point - min_point + 1;

  gtk_layout_set_size(GTK_LAYOUT(track_list_right), main_mixer->gui_right_size, main_mixer->gui_height + 5);
  gtk_layout_set_size(GTK_LAYOUT(marker_track_list_right), main_mixer->gui_right_size, MARK_HEIGHT);

  mark_gui_replace_marks();
}

void main_gui_redraw_track_view(void)
{
  /* some value of 1 channel has changed */
  /* since we don't know what track it is in, we redraw all */
  /* yes, it is barbarian */
  /* we simply invalidate all the drawing_area */
  int i;

  for (i=0; i<main_mixer->nb_tracks; i++)
    /* when generating view, drawing_area may not exist yet and this function is called, so we check */
    if (main_mixer->tracks[i]->drawing_area)
      gtk_widget_queue_draw(main_mixer->tracks[i]->drawing_area);
}

void main_gui_add_track(track *t)
{
  GtkRequisition req;
  GtkWidget *draw;
  channel *input, *posi = 0 /* for no gcc warning */;
  int y;

  input = t->channel;
  while (input->c.type != CHANNEL_TYPE_INPUT) {
    if (input->c.type == CHANNEL_TYPE_POS)
      posi = input;
    input = input->c.subchannel;
  }

  generate_view(t);

  /* We suppose that added track will go to the end of the layout.
   * If Songs is hacked in strange ways, one might need to recode
   * this function.
   */

  y = main_mixer->gui_height;

  /* updating size of left and right part */
  /* (the right width is not correct yet, it will be fixed by main_gui_update_track_view(); */
  gtk_widget_size_request(t->box, &req);

  if (req.height > 64+10)
    main_mixer->gui_height += 5 + req.height;
  else
    main_mixer->gui_height += 5 + 64+10;

  if (req.width > main_mixer->gui_width)
    main_mixer->gui_width = req.width;

  gtk_layout_set_size(GTK_LAYOUT(track_list_left), main_mixer->gui_width+10, main_mixer->gui_height + 5);
  gtk_layout_set_size(GTK_LAYOUT(track_list_right), main_mixer->gui_right_size, main_mixer->gui_height + 5);
  gtk_layout_set_size(GTK_LAYOUT(marker_track_list_right), main_mixer->gui_right_size, MARK_HEIGHT);

  gtk_layout_put(GTK_LAYOUT(track_list_left), t->box, 5, y+5);

  t->drawing_y = y+5;
  t->drawing_x = (((pos *)posi)->global_start-main_mixer->minimum_sample+4095)/4096;

  t->drawing_area = draw = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(draw), (input->c.last_point(input)+4095) / 4096 , 64);
  gtk_widget_modify_bg(draw, GTK_STATE_NORMAL, &draw->style->white);

  gtk_widget_add_events(t->drawing_area,
             GDK_ENTER_NOTIFY_MASK |
             GDK_LEAVE_NOTIFY_MASK |
             GDK_POINTER_MOTION_MASK |
             GDK_BUTTON_PRESS_MASK |
             GDK_BUTTON_RELEASE_MASK);

  g_signal_connect_swapped(t->drawing_area, "motion-notify-event", G_CALLBACK(track_pointer_motion_event), t);
  g_signal_connect_swapped(t->drawing_area, "enter-notify-event", G_CALLBACK(enter_track_window), t);
  g_signal_connect_swapped(t->drawing_area, "leave-notify-event", G_CALLBACK(leave_track_window), t);
  g_signal_connect_swapped(t->drawing_area, "button-press-event", G_CALLBACK(track_button_press_event), t);
  g_signal_connect_swapped(t->drawing_area, "button-release-event", G_CALLBACK(track_button_release_event), t);

  view_set_callbacks(t);

  gtk_layout_put(GTK_LAYOUT(track_list_right), draw, t->drawing_x, t->drawing_y);
  gtk_widget_show(draw);

  main_gui_update_track_view();

  {
    GtkRequisition req;
    gtk_widget_size_request(left_box, &req);
    gtk_paned_set_position(GTK_PANED(panned), main_mixer->gui_width+10 > req.width ? main_mixer->gui_width+10: req.width);
  }

  t->gui_on = 1;
}

void main_gui_sub_track(track *d)
{
  int i;
  GtkRequisition req;
  int y;
  track *t;

  /* to be simple, we move all the remaining tracks to their new position (some may not move, but it's simpler
   * this way, and we can get the real width and height of the layout)
   */

  y = 0;

  main_mixer->gui_width = 1;

  for (i=0; i<main_mixer->nb_tracks; i++) {
    t = main_mixer->tracks[i];

    gtk_widget_size_request(t->box, &req);

    if (req.width > main_mixer->gui_width)
      main_mixer->gui_width = req.width;

    gtk_layout_move(GTK_LAYOUT(track_list_left), t->box, 5, y + 5);

    gtk_layout_move(GTK_LAYOUT(track_list_right), t->drawing_area, t->drawing_x, y + 5);
    t->drawing_y = y + 5;

    y += req.height + 5;
  }

  main_mixer->gui_height = y;

  /* the right window is resized inside this call */
  main_gui_update_track_view();

  gtk_layout_set_size(GTK_LAYOUT(track_list_left), main_mixer->gui_width+10, y + 5);

  {
    GtkRequisition req;
    gtk_widget_size_request(left_box, &req);
    gtk_paned_set_position(GTK_PANED(panned), main_mixer->gui_width+10 > req.width ? main_mixer->gui_width+10: req.width);
  }

  gtk_widget_destroy(d->drawing_area);
}

static GList *start_list = 0;

void main_gui_update_start_play_list(void)
{
  GList *new;
  int i;

  if (start_list) {
    g_list_free(start_list);
    start_list = 0;
  }

  start_list = g_list_append(start_list, "First audio sample");
  if (!start_list) {
    oom();
    display_error();
    return;
  }

  for (i=0; i<mark_list_size; i++) {
    new = g_list_append(start_list, mark_list[i]->name);
    if (!new) {
      oom();
      display_error();
      g_list_free(start_list);
      start_list = 0;
      return;
    }
    start_list = new;
  }

  /* error used as buffer */
  reset_error();
  add_error_char((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry)));

  gtk_combo_set_popdown_strings(GTK_COMBO(main_combo_play_start), start_list);

  if (!get_mark(get_error())) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry), "");
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry), "First audio sample");
  } else {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry), "");
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry), get_error());
  }
}

static GList *stop_list = 0;

void main_gui_update_stop_play_list(void)
{
  GList *new;
  int i;

  if (stop_list) {
    g_list_free(stop_list);
    stop_list = 0;
  }

  stop_list = g_list_append(stop_list, "Infinity");
  if (!stop_list) {
    oom();
    display_error();
    return;
  }

  new = g_list_append(stop_list, "Last audio sample");
  if (!new) {
    g_list_free(stop_list);
    stop_list = 0;
    oom();
    display_error();
    return;
  }
  stop_list = new;

  for (i=0; i<mark_list_size; i++) {
    new = g_list_append(stop_list, mark_list[i]->name);
    if (!new) {
      oom();
      display_error();
      g_list_free(stop_list);
      stop_list = 0;
      return;
    }
    stop_list = new;
  }

  /* error used as buffer */
  reset_error();
  add_error_char((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry)));

  gtk_combo_set_popdown_strings(GTK_COMBO(main_combo_play_stop), stop_list);

  if (!get_mark(get_error())) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), "");
    if (!strcmp(get_error(), "Last audio sample"))
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), "Last audio sample");
    else
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), "Infinity");
  } else {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), "");
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), get_error());
  }
}

static GList *marker_list = 0;

void main_gui_update_marker_list(void)
{
  GList *new;
  int i;

  if (marker_list) {
    g_list_free(marker_list);
    marker_list = 0;
  }

  for (i=0; i<mark_list_size; i++) {
    new = g_list_append(marker_list, mark_list[i]->name);
    if (!new) {
      oom();
      display_error();
      g_list_free(marker_list);
      marker_list = 0;
      return;
    }
    marker_list = new;
  }

  if (!marker_list) {
    marker_list = g_list_append(0, "");
    if (!marker_list) {
      oom();
      display_error();
      return;
    }
  }

  /* error used as buffer */
  reset_error();
  add_error_char((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry)));

  gtk_combo_set_popdown_strings(GTK_COMBO(main_combo_marker_list), marker_list);

  if (!get_mark(get_error())) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry), "");
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry), marker_list->data);
  } else {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry), "");
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry), get_error());
  }
}

static int set_left_paned(void)
{
  gtk_paned_set_position(GTK_PANED(panned_right), gtk_paned_get_position(GTK_PANED(panned_left)));
  return FALSE;
}

static int set_right_paned(void)
{
  gtk_paned_set_position(GTK_PANED(panned_left), gtk_paned_get_position(GTK_PANED(panned_right)));
  return FALSE;
}

void main_gui(void)
{
  GtkAccelGroup *main_accel_group;
  GtkWidget *main_vbox;
  GtkWidget *main_menubar;
  GtkWidget *main_menubar_file;
  GtkWidget *main_menubar_tool;
  GtkWidget *main_menubar_help;
  GtkWidget *menu_file;
  GtkWidget *menu_file_new;
  GtkWidget *menu_file_open;
  GtkWidget *menu_file_save;
  GtkWidget *menu_file_saveas;
  GtkWidget *menu_file_setup;
  GtkWidget *menu_file_quit;
  GtkWidget *menu_tool;
  GtkWidget *menu_tool_newtrack;
  GtkWidget *menu_tool_muteall;
  GtkWidget *menu_tool_mutenone;
  GtkWidget *menu_tool_genall;
  GtkWidget *menu_tool_gennone;
  GtkWidget *menu_tool_newmark;
  GtkWidget *menu_tool_marks_on;
  GtkWidget *menu_tool_marks_off;
  GtkWidget *menu_tool_play;
  GtkWidget *menu_tool_stop;
  GtkWidget *menu_tool_record;
  GtkWidget *menu_tool_dump;
  GtkWidget *menu_tool_effects;
  GtkWidget *menu_tool_mixpreview;
#if 0
  GtkWidget *menu_subtool_realtime;
  GtkWidget *menu_subtool_precalc;
  GtkWidget *realtime_menu;
  GtkWidget *precalc_menu;
#endif
  GtkWidget *menu_help;
  GtkWidget *menu_help_help;
  GtkWidget *menu_help_about;
  GtkWidget *main_bottom;

  current_file[0] = 0;

  /* some cursors */
  track_move = gdk_cursor_new(GDK_FLEUR);
  track_left_arrow = gdk_cursor_new(GDK_SB_LEFT_ARROW);
  track_right_arrow = gdk_cursor_new(GDK_SB_RIGHT_ARROW);

  /* creating objects */
  main_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  {
    /* let's force its minimal size to be 1x1, I don't like to let the user with a too big window if she wants to resize it smaller */
    GdkGeometry g;
    g.min_width = g.min_height = 1;
    gtk_window_set_geometry_hints(GTK_WINDOW(main_win), main_win, &g, GDK_HINT_MIN_SIZE);
  }
  gtk_widget_set_size_request(main_win, 700, 500);
  /* error is used now as a buffer, to set the window's title */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  gtk_window_set_title(GTK_WINDOW(main_win), get_error());

  main_vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(main_win), main_vbox);

  /* menu */
  main_menubar = gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(main_vbox), main_menubar, FALSE, FALSE, 0);
  main_menubar_file = gtk_menu_item_new_with_mnemonic("_File");
  main_menubar_tool = gtk_menu_item_new_with_mnemonic("_Tool");
  main_menubar_help = gtk_menu_item_new_with_mnemonic("_Help");
  gtk_menu_shell_append(GTK_MENU_SHELL(main_menubar), main_menubar_file);
  gtk_menu_shell_append(GTK_MENU_SHELL(main_menubar), main_menubar_tool);
  gtk_menu_shell_append(GTK_MENU_SHELL(main_menubar), main_menubar_help);

  menu_file = gtk_menu_new();
  menu_file_new = gtk_menu_item_new_with_mnemonic("_New");
  menu_file_open = gtk_menu_item_new_with_mnemonic("_Open");
  menu_file_save = gtk_menu_item_new_with_mnemonic("_Save");
  menu_file_saveas = gtk_menu_item_new_with_mnemonic("Save _as");
  menu_file_setup = gtk_menu_item_new_with_mnemonic("_Preferences");
  menu_file_quit = gtk_menu_item_new_with_mnemonic("_Quit");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menubar_file), menu_file);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_new);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_open);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_save);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_saveas);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_setup);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), menu_file_quit);

  menu_tool = gtk_menu_new();
  menu_tool_newtrack = gtk_menu_item_new_with_mnemonic("_New track");
  menu_tool_muteall = gtk_menu_item_new_with_mnemonic("M_ute all");
  menu_tool_mutenone = gtk_menu_item_new_with_mnemonic("Mu_te none");
  menu_tool_genall = gtk_menu_item_new_with_mnemonic("_View all tracks");
  menu_tool_gennone = gtk_menu_item_new_with_mnemonic("V_iew no track");
  menu_tool_newmark = gtk_menu_item_new_with_mnemonic("New _mark");
  menu_tool_marks_on = gtk_menu_item_new_with_mnemonic("All marks _on");
  menu_tool_marks_off = gtk_menu_item_new_with_mnemonic("All marks o_ff");
  menu_tool_play = gtk_menu_item_new_with_mnemonic("_Play");
  menu_tool_stop = gtk_menu_item_new_with_mnemonic("_Stop");
  menu_tool_record = gtk_menu_item_new_with_mnemonic("_Record");
  menu_tool_dump = gtk_menu_item_new_with_mnemonic("_Dump");
  menu_tool_effects = gtk_menu_item_new_with_mnemonic("_Effects");
  menu_tool_mixpreview = gtk_menu_item_new_with_mnemonic("Mi_x preview");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menubar_tool), menu_tool);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_newtrack);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_muteall);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_mutenone);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_genall);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_gennone);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_newmark);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_marks_on);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_marks_off);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_play);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_stop);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_record);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_dump);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_effects);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_tool_mixpreview);

#if 0
  menu_subtool_realtime = gtk_menu_item_new_with_mnemonic("R_ealtime effects");
  menu_subtool_precalc = gtk_menu_item_new_with_mnemonic("N_on realtime effects");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_subtool_realtime);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_tool), menu_subtool_precalc);

  realtime_menu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_subtool_realtime), realtime_menu);
  realtime_vol = gtk_menu_item_new_with_mnemonic("_Gain");
  realtime_pan = gtk_menu_item_new_with_mnemonic("_Panning");
  realtime_volenv = gtk_menu_item_new_with_mnemonic("G_ain envelop");
  realtime_panenv = gtk_menu_item_new_with_mnemonic("Pa_nning envelop");
  realtime_eq = gtk_menu_item_new_with_mnemonic("_Equalizer");
  realtime_compressor = gtk_menu_item_new_with_mnemonic("_Compressor");
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_vol);
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_pan);
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_volenv);
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_panenv);
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_eq);
  gtk_menu_shell_append(GTK_MENU_SHELL(realtime_menu), realtime_compressor);

  precalc_menu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_subtool_precalc), precalc_menu);
  precalc_reverb = gtk_menu_item_new_with_mnemonic("_Reverb (mono)");
  precalc_reverb_stereo = gtk_menu_item_new_with_mnemonic("R_everb (stereo)");
  gtk_menu_shell_append(GTK_MENU_SHELL(precalc_menu), precalc_reverb);
  gtk_menu_shell_append(GTK_MENU_SHELL(precalc_menu), precalc_reverb_stereo);
#endif

  menu_help = gtk_menu_new();
  menu_help_help = gtk_menu_item_new_with_mnemonic("_Help");
  menu_help_about = gtk_menu_item_new_with_mnemonic("_About");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menubar_help), menu_help);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), menu_help_help);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), menu_help_about);

  /* play handling line */
{
  GtkWidget *box;
  GtkWidget *play, *stop, *record;

  box = gtk_hbox_new(FALSE, 10);
  gtk_container_set_border_width(GTK_CONTAINER(box), 3);

  gtk_box_pack_start(GTK_BOX(main_vbox), box, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(box), gtk_label_new("Play from:"), FALSE, FALSE, 0);
  main_combo_play_start = gtk_combo_new();
  gtk_box_pack_start(GTK_BOX(box), main_combo_play_start, FALSE, FALSE, 0);
  main_gui_update_start_play_list();
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(main_combo_play_start)->entry), FALSE);

  gtk_box_pack_start(GTK_BOX(box), gtk_label_new("to:"), FALSE, FALSE, 0);
  main_combo_play_stop = gtk_combo_new();
  gtk_box_pack_start(GTK_BOX(box), main_combo_play_stop, FALSE, FALSE, 0);
  main_gui_update_stop_play_list();
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(main_combo_play_stop)->entry), FALSE);

  play = gtk_button_new_with_label("Play");
  stop = gtk_button_new_with_label("Stop");
  record = gtk_button_new_with_label("Record");

  gtk_box_pack_start(GTK_BOX(box), play, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), stop, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), record, FALSE, FALSE, 0);

  g_signal_connect(play, "clicked", G_CALLBACK(play_event), 0);
  g_signal_connect(stop, "clicked", G_CALLBACK(stop_event), 0);
  g_signal_connect(record, "clicked", G_CALLBACK(record_event), 0);
}

  /* the toolbox */
{
  GtkWidget *toolbox = create_toolbox();

  gtk_box_pack_start(GTK_BOX(main_vbox), toolbox, FALSE, FALSE, 0);
}

  /* track list area */
{
  GtkWidget *box1, *box2;
  GtkWidget *scrollbar_left_horiz = gtk_hscrollbar_new(0);
  GtkWidget *scrollbar_right_vert_2 = gtk_vscrollbar_new(0);
  GtkWidget *frame;
  GtkWidget *hbox;
  GtkWidget *go_to, *edit, *delete;

  scrollbar_right_vert = gtk_vscrollbar_new(0);
  scrollbar_right_horiz = gtk_hscrollbar_new(0);

  gtk_range_set_increments(GTK_RANGE(scrollbar_left_horiz), 10, 100);
  gtk_range_set_increments(GTK_RANGE(scrollbar_right_horiz), 10, 100);
  gtk_range_set_increments(GTK_RANGE(scrollbar_right_vert), 10, 100);
  gtk_range_set_increments(GTK_RANGE(scrollbar_right_vert_2), 10, 100);

  panned_left = gtk_vpaned_new();
  panned_right = gtk_vpaned_new();

  /* a hack to let the two paned be at the same vertical position (if you know a better way, you're welcome) */
  g_signal_connect(panned_left, "motion-notify-event", G_CALLBACK(set_left_paned), 0);
  g_signal_connect(panned_right, "motion-notify-event", G_CALLBACK(set_right_paned), 0);
  g_signal_connect(panned_left, "button-release-event", G_CALLBACK(set_left_paned), 0);
  g_signal_connect(panned_right, "button-release-event", G_CALLBACK(set_right_paned), 0);

  panned = gtk_hpaned_new();
  gtk_box_pack_start(GTK_BOX(main_vbox), panned, TRUE, TRUE, 0);

  /* feeding left part */
  track_list_left = gtk_layout_new(
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_left_horiz)),
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_right_vert)));

  left_box = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(left_box), 5);

  main_combo_marker_list = gtk_combo_new();
  main_gui_update_marker_list();
  gtk_box_pack_start(GTK_BOX(left_box), main_combo_marker_list, FALSE, FALSE, 0);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry), FALSE);

  go_to = gtk_button_new_with_label("Go to");
  edit = gtk_button_new_with_label("Edit");
  delete = gtk_button_new_with_label("Delete");
  hbox = gtk_hbox_new(TRUE, 5);
  gtk_box_pack_start(GTK_BOX(left_box), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), go_to, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), edit, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), delete, TRUE, TRUE, 0);

  //gtk_paned_pack1(GTK_PANED(panned), panned_left, TRUE, TRUE);
  gtk_paned_add1(GTK_PANED(panned), panned_left);
  box1 = gtk_vbox_new(FALSE, 0);
  gtk_paned_pack1(GTK_PANED(panned_left), left_box, FALSE, FALSE);
  gtk_paned_add2(GTK_PANED(panned_left), box1);

  frame = gtk_frame_new(0);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  gtk_container_add(GTK_CONTAINER(frame), track_list_left);

  gtk_box_pack_start(GTK_BOX(box1), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box1), scrollbar_left_horiz, FALSE, FALSE, 0);

  /* feeding right part */
  track_list_right = gtk_layout_new(
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_right_horiz)),
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_right_vert)));

  gtk_widget_add_events(track_list_right, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK);

  marker_track_list_right = gtk_layout_new(
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_right_horiz)),
	gtk_range_get_adjustment(GTK_RANGE(scrollbar_right_vert_2)));

  //gtk_widget_add_events(marker_track_list_right, GDK_BUTTON_PRESS_MASK);

  gtk_paned_add2(GTK_PANED(panned), panned_right);
  box1 = gtk_vbox_new(FALSE, 0);
  gtk_paned_add2(GTK_PANED(panned_right), box1);

  frame = gtk_frame_new(0);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  box2 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box2), scrollbar_right_vert_2, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame), marker_track_list_right);
  gtk_paned_add1(GTK_PANED(panned_right), box2);

  frame = gtk_frame_new(0);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_container_add(GTK_CONTAINER(frame), track_list_right);

  box2 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box1), scrollbar_right_horiz, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box2), scrollbar_right_vert, FALSE, FALSE, 0);

  gtk_widget_set_size_request(track_list_left, 1, 1);
  gtk_widget_set_size_request(track_list_right, 1, 1);
  gtk_widget_set_size_request(marker_track_list_right, 1, MARK_HEIGHT);
  gtk_layout_set_size(GTK_LAYOUT(marker_track_list_right), 1, MARK_HEIGHT);

  g_signal_connect(go_to, "clicked", G_CALLBACK(mark_gui_goto), 0);
  g_signal_connect(edit, "clicked", G_CALLBACK(mark_gui_edit), 0);
  g_signal_connect(delete, "clicked", G_CALLBACK(mark_gui_delete), 0);

  g_signal_connect(track_list_right, "enter-notify-event", G_CALLBACK(enter_track_window), 0);
  g_signal_connect(track_list_right, "leave-notify-event", G_CALLBACK(leave_track_window), 0);
  g_signal_connect(track_list_right, "motion-notify-event", G_CALLBACK(track_pointer_motion_event), 0);
  g_signal_connect(track_list_right, "button-press-event", G_CALLBACK(track_button_press_event), 0);
  g_signal_connect(track_list_right, "scroll-event", G_CALLBACK(track_scroll_event), 0);
  g_signal_connect(track_list_left, "scroll-event", G_CALLBACK(track_scroll_event), 0);
  g_signal_connect(marker_track_list_right, "button-press-event", G_CALLBACK(marker_track_button_press_event), 0);
}

  /* bottom information line */
  main_bottom = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_vbox), main_bottom, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_bottom), gtk_label_new("This information line is unused yet, if you have ideas about it, contact the authors."), FALSE, FALSE, 0);

  /* accel stuff for the menu */
  main_accel_group = gtk_accel_group_new();

  gtk_accel_map_add_entry("<Songs>/File/New", GDK_n, GDK_CONTROL_MASK);
  gtk_accel_map_add_entry("<Songs>/File/Open", GDK_o, GDK_CONTROL_MASK);
  gtk_accel_map_add_entry("<Songs>/File/Save", GDK_s, GDK_CONTROL_MASK);
  gtk_accel_map_add_entry("<Songs>/File/Preferences", GDK_p, GDK_CONTROL_MASK);
  gtk_accel_map_add_entry("<Songs>/File/Quit", GDK_q, GDK_CONTROL_MASK);

  gtk_accel_map_add_entry("<Songs>/Tool/New track", GDK_n, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Mute all", GDK_u, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Mute none", GDK_u, GDK_MOD1_MASK|GDK_SHIFT_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/View all tracks", GDK_v, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/View no track", GDK_v, GDK_MOD1_MASK|GDK_SHIFT_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/New mark", GDK_a, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/All marks on", GDK_O, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/All marks off", GDK_O, GDK_MOD1_MASK|GDK_SHIFT_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Play", GDK_p, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Stop", GDK_s, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Record", GDK_r, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Dump", GDK_d, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Effects", GDK_e, GDK_MOD1_MASK);
  gtk_accel_map_add_entry("<Songs>/Tool/Mix preview", GDK_x, GDK_MOD1_MASK);

  gtk_accel_map_add_entry("<Songs>/Help/Help", GDK_h, GDK_CONTROL_MASK);

  gtk_window_add_accel_group(GTK_WINDOW(main_win), main_accel_group);
  gtk_menu_set_accel_group(GTK_MENU(menu_file), main_accel_group);
  gtk_menu_set_accel_group(GTK_MENU(menu_tool), main_accel_group);
  gtk_menu_set_accel_group(GTK_MENU(menu_help), main_accel_group);

  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_new), "<Songs>/File/New");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_open), "<Songs>/File/Open");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_save), "<Songs>/File/Save");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_saveas), "<Songs>/File/Save As");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_setup), "<Songs>/File/Preferences");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_file_quit), "<Songs>/File/Quit");

  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_newtrack), "<Songs>/Tool/New track");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_muteall), "<Songs>/Tool/Mute all");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_mutenone), "<Songs>/Tool/Mute none");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_genall), "<Songs>/Tool/View all tracks");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_gennone), "<Songs>/Tool/View no track");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_newmark), "<Songs>/Tool/New mark");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_marks_on), "<Songs>/Tool/All marks on");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_marks_off), "<Songs>/Tool/All marks off");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_play), "<Songs>/Tool/Play");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_stop), "<Songs>/Tool/Stop");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_record), "<Songs>/Tool/Record");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_dump), "<Songs>/Tool/Dump");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_effects), "<Songs>/Tool/Effects");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_tool_mixpreview), "<Songs>/Tool/Mix preview");

  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_help_help), "<Songs>/Help/Help");
  gtk_menu_item_set_accel_path(GTK_MENU_ITEM(menu_help_about), "<Songs>/Help/About");

  /* handling events */
  g_signal_connect(main_win, "delete_event", G_CALLBACK(main_quit_event), 0);

  g_signal_connect(menu_file_new, "activate", G_CALLBACK(new_event), 0);
  g_signal_connect(menu_file_open, "activate", G_CALLBACK(open_event), 0);
  g_signal_connect(menu_file_save, "activate", G_CALLBACK(save_event), 0);
  g_signal_connect(menu_file_saveas, "activate", G_CALLBACK(saveas_event), 0);
  g_signal_connect(menu_file_setup, "activate", G_CALLBACK(setup), 0);
  g_signal_connect(menu_file_quit, "activate", G_CALLBACK(main_quit_event), 0);

  g_signal_connect(menu_tool_newtrack, "activate", G_CALLBACK(newtrack_event), 0);
  g_signal_connect(menu_tool_muteall, "activate", G_CALLBACK(mute_all), 0);
  g_signal_connect(menu_tool_mutenone, "activate", G_CALLBACK(mute_none), 0);
  g_signal_connect(menu_tool_genall, "activate", G_CALLBACK(generate_view_all), 0);
  g_signal_connect(menu_tool_gennone, "activate", G_CALLBACK(generate_view_none), 0);

  g_signal_connect(menu_tool_newmark, "activate", G_CALLBACK(mark_gui_new), 0);
  g_signal_connect(menu_tool_marks_on, "activate", G_CALLBACK(mark_gui_all_on), 0);
  g_signal_connect(menu_tool_marks_off, "activate", G_CALLBACK(mark_gui_all_off), 0);

  g_signal_connect(menu_tool_play, "activate", G_CALLBACK(play_event), 0);
  g_signal_connect(menu_tool_stop, "activate", G_CALLBACK(stop_event), 0);
  g_signal_connect(menu_tool_record, "activate", G_CALLBACK(record_event), 0);
  g_signal_connect(menu_tool_dump, "activate", G_CALLBACK(dump_mix), 0);


  g_signal_connect(menu_tool_effects, "activate", G_CALLBACK(effects_event), 0);
  g_signal_connect(menu_tool_mixpreview, "activate", G_CALLBACK(mix_preview_on), 0);

  g_signal_connect(menu_help_help, "activate", G_CALLBACK(help_gui), 0);
  g_signal_connect(menu_help_about, "activate", G_CALLBACK(about_gui), 0);

  /* ok */
  gtk_widget_show_all(main_win);

  /* a red gc */
{
  GdkColor col;

  col.red = 65535;
  col.green = 0;
  col.blue = 0;

  if (gdk_colormap_alloc_color(gdk_colormap_get_system(), &col, FALSE, TRUE) == FALSE) {
    gc_err:
    reset_error();
    add_error_char("Unable to allocate color red, some visual effects will look black instead of red.\n");
    display_error();
    red_gc = main_win->style->black_gc;
  } else {
    red_gc = gdk_gc_new(main_win->window);
    if (!red_gc)
      goto gc_err;
    gdk_gc_copy(red_gc, main_win->style->black_gc);
    gdk_gc_set_foreground(red_gc, &col);
  }
}

  /* we have to synchronize the 2 paned, so let's say right size will be the one of left */
  set_left_paned();
}

/* I had to put this function somewhere (in a file that include <gtk/gtk.h>, so why not here, huh?) */
void set_label_text(void *label, char *text)
{
  gtk_label_set_text(GTK_LABEL(label), text);
}
