/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

/* all this code is a big hack, maybe there is better way to handle this
 * (gtk-ly speaking I mean)
 */

#include "mark_gui.h"

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "mixer.h"
#include "error.h"
#include "record_gui.h"
#include "main_gui.h"
#include "version.h"
#include "yes_no_gui.h"

static int events(mark *m, GdkEvent *ev)
{
  if (ev->type == GDK_KEY_PRESS || ev->type == GDK_KEY_RELEASE)
    return TRUE;

  if (ev->type != GDK_MOTION_NOTIFY)
    return FALSE;

  m->mark_set(m, m->last_sample + (ev->motion.x_root - m->last_mouse_x) * 4096, m->y_last_position + ev->motion.y_root - m->last_mouse_y);

  return TRUE;
}

static int pickup_press(mark *m, GdkEventButton *b)
{
  if (b->type != GDK_BUTTON_PRESS || b->button != 1)
    return FALSE;

  m->last_mouse_x = b->x_root;
  m->last_mouse_y = b->y_root;
  m->last_sample = m->sample;
  m->y_last_position = m->y_position;

  return FALSE;
}

static void pressed_event(mark *m)
{
  gtk_grab_add(m->gui);

  m->ev = g_signal_connect_swapped(m->gui, "event", G_CALLBACK(events), m);
}

static void released_event(mark *m)
{
  gtk_grab_remove(m->gui);

  g_signal_handler_disconnect(m->gui, m->ev);
}

static void create_gui(mark *m)
{
  int pos;
  extern GtkWidget *marker_track_list_right;

  pos = m->sample - main_mixer->minimum_sample;
  pos /= 4096;

  m->gui = gtk_button_new();

  if (m->name)
    m->gui_text = gtk_label_new(m->name);

  /* we must do that now, it won't work after realization (even if it is supposed to... I love the gtk docs... really... no joke...) */
  gtk_widget_add_events(m->gui, GDK_POINTER_MOTION_MASK);

  gtk_widget_set_size_request(m->gui, 10, 10);
  gtk_layout_put(GTK_LAYOUT(marker_track_list_right), (GtkWidget *)m->gui, pos, m->y_position);
  if (m->name)
    gtk_layout_put(GTK_LAYOUT(marker_track_list_right), (GtkWidget *)m->gui_text, pos, m->y_position + 10);
  gtk_widget_show_all((GtkWidget *)m->gui);
  if (m->name)
    gtk_widget_show_all((GtkWidget *)m->gui_text);

  g_signal_connect_swapped(m->gui, "pressed", G_CALLBACK(pressed_event), m);
  g_signal_connect_swapped(m->gui, "released", G_CALLBACK(released_event), m);
  g_signal_connect_swapped(m->gui, "button-press-event", G_CALLBACK(pickup_press), m);
}

static void delete_gui(mark *m)
{
  gtk_widget_hide(m->gui);
  gtk_widget_destroy((GtkWidget *)m->gui);
  if (m->name) {
    gtk_widget_hide(m->gui_text);
    gtk_widget_destroy((GtkWidget *)m->gui_text);
  }
}

static void internal_play_mark_set(mark *, int, int);

static mark play_mark = {on:0, name:0, gui:0, mark_set:internal_play_mark_set, y_position:5};

static void internal_play_mark_set(mark *m, int sample, int y)
{
  if (y < 0) y = 0;
  if (y > MARK_HEIGHT - 10) y = MARK_HEIGHT - 10;

  play_mark.y_position = y;

  main_mixer->next_sample = sample;
  play_mark_set(sample);
}

void play_mark_on(int sample)
{
  if (play_mark.on)
    return;

  play_mark.sample = sample;

  create_gui(&play_mark);

  play_mark.on = 1;
}

void play_mark_off(void)
{
  if (!play_mark.on)
    return;

  delete_gui(&play_mark);

  play_mark.on = 0;
}

void play_mark_set(int sample)
{
  int pos;
  extern GtkWidget *marker_track_list_right;

  play_mark.sample = sample;

  if (!play_mark.on)
    return;

  pos = play_mark.sample - main_mixer->minimum_sample;
  pos /= 4096;

  gtk_layout_move(GTK_LAYOUT(marker_track_list_right), (GtkWidget *)play_mark.gui, pos, play_mark.y_position);
}

static void mark_list_changed(int line)
{
  /* line 'line' has changed (added or removed), let the ones who use the list know this fact */
  main_gui_update_start_play_list();
  main_gui_update_stop_play_list();
  main_gui_update_marker_list();

  record_gui_update_start_record_list();
}

mark **mark_list = 0;
int mark_list_size = 0;
int mark_list_maxsize = 0;

static int add_to_list(mark *m)
{
  if (mark_list_size == mark_list_maxsize) {
    mark **n = realloc(mark_list, sizeof(mark *) * (mark_list_maxsize + 32));
    if (!n) {
      oom();
      display_error();
      return -1;
    }
    mark_list = n;
    mark_list_maxsize += 32;
  }

  mark_list[mark_list_size] = m;

  mark_list_size++;

  /* we notify all the people who are using the list that it has been modified */
  mark_list_changed(mark_list_size-1);

  return 0;
}

static void remove_from_list(mark *m)
{
  int i;
  int changed;

  for (i=0; i<mark_list_size; i++)
    if (mark_list[i] == m)
      break;

  if (i == mark_list_size) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": internal error, contact the authors.\n");
    display_error();
    return;
  }

  changed = i;

  for (; i<mark_list_size-1; i++)
    mark_list[i] = mark_list[i+1];

  mark_list_size--;

  /* try to free some memory */
  if (mark_list_maxsize - mark_list_size > 32) {
    mark **n = realloc(mark_list, (mark_list_maxsize-32) * sizeof(mark *));
    /* silently ignore errors */
    if (n) {
      mark_list = n;
      mark_list_maxsize -= 32;
    }
  }

  /* we notify all the people who are using the list that it has been modified */
  mark_list_changed(mark_list_size-1);
}

void mark_gui_replace_marks(void)
{
  extern GtkWidget *marker_track_list_right;
  int i;
  mark *m;
  int pos;

  for (i=0; i<mark_list_size; i++) {
    m = mark_list[i];
    pos = m->sample - main_mixer->minimum_sample;
    pos /= 4096;
    gtk_layout_move(GTK_LAYOUT(marker_track_list_right), m->gui, pos, m->y_position);
    gtk_layout_move(GTK_LAYOUT(marker_track_list_right), m->gui_text, pos, m->y_position + 10);
  }
}

static void mark_set(mark *m, int sample, int y)
{
  int pos;
  extern GtkWidget *marker_track_list_right;

  if (y < 0) y = 0;
  if (y > MARK_HEIGHT - 10) y = MARK_HEIGHT - 10;

  m->sample = sample;
  m->y_position = y;

  if (m->gui_edit_on) {
    char zo[64];
    sprintf(zo, "%d", m->sample);
    gtk_entry_set_text(GTK_ENTRY(m->gui_edit_position), zo);
  }

  if (!m->on)
    return;

  pos = m->sample - main_mixer->minimum_sample;
  pos /= 4096;

  gtk_layout_move(GTK_LAYOUT(marker_track_list_right), m->gui, pos, m->y_position);
  gtk_layout_move(GTK_LAYOUT(marker_track_list_right), m->gui_text, pos, m->y_position + 10);

  modify = 1;
}

mark *new_mark(char *name, int sample)
{
  mark *m;

  if (!name) name = "Marker";

  m = malloc(sizeof(mark));
  if (!m)
    return 0;
  memset(m, 0, sizeof(mark));

  m->sample = sample;
  m->y_position = 5;

  m->name = strdup(name);
  if (!m->name) {
    free(m);
    return 0;
  }

  m->mark_set = mark_set;

  if (add_to_list(m) == -1) {
    free(m->name);
    free(m);
    return 0;
  }

  create_gui(m);

  m->on = 1;

  modify = 1;

  return m;
}

void delete_mark(mark *m)
{
  if (m->on)
    delete_gui(m);
  m->on = 0;

  remove_from_list(m);

  free(m->name);
  free(m);

  modify = 1;
}

void mark_on(mark *m)
{
  if (m->on)
    return;

  create_gui(m);

  m->on = 1;

  modify = 1;
}

void mark_off(mark *m)
{
  if (!m->on)
    return;

  delete_gui(m);

  m->on = 0;

  modify = 1;
}

mark *get_mark(char *name)
{
  int i;

  for (i=0; i<mark_list_size; i++)
    if (!strcmp(name, mark_list[i]->name))
      return mark_list[i];

  return 0;
}

mark *get_nth_mark(int n)
{
  return mark_list[n];
}

static void destroy_event(mark *m)
{
  m->gui_edit_on = 0;
}

static void gui_edit_error(int pos, char *pos_val, int name, char *name_val)
{
  reset_error();

  if (pos) {
    add_error_char("The position ");
    add_error_char(pos_val);
    add_error_char(" is not a valid position (it must be a number)\n");
  }

  if (name) {
    add_error_char("The name '");
    add_error_char(name_val);
    add_error_char(name == 1 ? "' is already in use.\n" : "' is invalid (hint: it may have leading spaces, which is not allowed).\n");
  }

  display_error();
}

static int update_verify(mark *m)
{
  long v;
  char *end;
  char *c1 = (char *)gtk_entry_get_text(GTK_ENTRY(m->gui_edit_name));
  char *c2 = (char *)gtk_entry_get_text(GTK_ENTRY(m->gui_edit_position));
  char *new_name;
  int no_good_name = 0;
  int no_good_position = 0;
  int i;
  int len;
  int line_in_list = 0 /* for no gcc warning */;

  v = strtol(c2, &end, 10);
  if (*c2 == 0 || *end != 0)
    no_good_position = 1;

  if ((len=strlen(c1)) == 0)
    no_good_name = 2;

  for (i=0; i<len; i++)
    if (!isspace(c1[i]))
      break;
  if (i == len)
    no_good_name = 2;

  if (isspace(c1[0]))
    no_good_name = 2;

  if (!no_good_name)
    if (!strcmp(c1, "First audio sample") ||
        !strcmp(c1, "Last audio sample") ||
        !strcmp(c1, "Infinity"))
      no_good_name = 1;

  if (!no_good_name)
    for (i = 0; i<mark_list_size; i++) {
      if (mark_list[i] == m) {
        line_in_list = i;
        continue;
      }
      if (!strcmp(c1, mark_list[i]->name))
        no_good_name = 1;
    }

  if (no_good_position || no_good_name) {
    gui_edit_error(no_good_position, c2, no_good_name, c1);
    return -1;
  }

  new_name = strdup(c1);
  if (!new_name) {
    oom();
    display_error();
    return -1;
  }

  free(m->name);
  m->name = new_name;

  mark_set(m, v, m->y_position);
  if (m->on)
    gtk_label_set_text(m->gui_text, new_name);
  mark_list_changed(line_in_list);

  return 0;
}

static void ok_verify(mark *m)
{
  if (update_verify(m) == -1)
    return;

  gtk_widget_destroy(m->gui_edit);
}

static void new_ok_verify(mark *m)
{
  long v;
  char *end;
  char *c1 = (char *)gtk_entry_get_text(GTK_ENTRY(m->gui_edit_name));
  char *c2 = (char *)gtk_entry_get_text(GTK_ENTRY(m->gui_edit_position));
  int no_good_name = 0;
  int no_good_position = 0;
  int i;
  int len;

  v = strtol(c2, &end, 10);
  if (*c2 == 0 || *end != 0)
    no_good_position = 1;

  if ((len=strlen(c1)) == 0)
    no_good_name = 2;

  for (i=0; i<len; i++)
    if (!isspace(c1[i]))
      break;
  if (i == len)
    no_good_name = 2;

  if (isspace(c1[0]))
    no_good_name = 2;

  if (!no_good_name)
    if (!strcmp(c1, "First audio sample") ||
        !strcmp(c1, "Last audio sample") ||
        !strcmp(c1, "Infinity"))
      no_good_name = 1;

  if (!no_good_name)
    for (i = 0; i<mark_list_size; i++) {
      if (!strcmp(c1, mark_list[i]->name)) {
        no_good_name = 1;
        break;
      }
    }

  if (no_good_position || no_good_name) {
    gui_edit_error(no_good_position, c2, no_good_name, c1);
    return;
  }

  new_mark(c1, v);

  gtk_widget_destroy(m->gui_edit);
}

static int new_in_use = 0;

static void new_destroy_event(void)
{
  new_in_use = 0;
}

static void show_hide_event(GtkToggleButton *b, mark *m)
{
  if (gtk_toggle_button_get_active(b) == TRUE)
    mark_on(m);
  else
    mark_off(m);
}

/* if mm == 0 we want to edit, if not it's new */
static void mark_gui_edit_or_new(mark *mm)
{
  char zo[64];
  extern GtkWidget *main_combo_marker_list;
  mark *m;
  GtkWidget *win;
  GtkWidget *vbox;
  GtkWidget *hbox1, *hbox2, *hbox3;
  GtkWidget *name;
  GtkWidget *position;
  GtkWidget *ok;
  GtkWidget *update = 0 /* for no gcc warnings */;
  GtkWidget *cancel;
  GtkWidget *show_hide = 0 /* for no gcc warnings */;

  if (mm)
    m = mm;
  else
    m = get_mark((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry)));

  if (!m)
    return;

  if (m->gui_edit_on)
    return;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_MOUSE);
  /* error used as buffer */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  if (!mm)
    add_error_char(" - Marker editor");
  else
    add_error_char(" - New marker");
  gtk_window_set_title(GTK_WINDOW(win), get_error());

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

  gtk_container_add(GTK_CONTAINER(win), vbox);

  hbox1 = gtk_hbox_new(FALSE, 5);
  hbox2 = gtk_hbox_new(FALSE, 5);
  hbox3 = gtk_hbox_new(TRUE, 5);

  gtk_box_pack_start(GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox3, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(hbox1), gtk_label_new("Name"), FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox2), gtk_label_new("Position"), FALSE, FALSE, 0);

  name = gtk_entry_new();
  position = gtk_entry_new();

  if (!mm)
    gtk_entry_set_text(GTK_ENTRY(name), m->name);

  sprintf(zo, "%d", m->sample);
  gtk_entry_set_text(GTK_ENTRY(position), zo);

  gtk_box_pack_start(GTK_BOX(hbox1), name, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox2), position, TRUE, TRUE, 0);

  if (!mm) {
    show_hide = gtk_check_button_new_with_mnemonic("_Show");
    gtk_box_pack_start(GTK_BOX(vbox), show_hide, FALSE, FALSE, 0);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_hide), m->on ? TRUE : FALSE);
  }

  ok = gtk_button_new_with_mnemonic("_Ok");
  if (!mm)
    update = gtk_button_new_with_mnemonic("_Update");
  cancel = gtk_button_new_with_mnemonic("_Cancel");

  if (!mm)
    gtk_box_pack_start(GTK_BOX(hbox3), update, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox3), ok, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox3), cancel, FALSE, FALSE, 0);

  /* callbacks */
  if (!mm) {
    g_signal_connect_swapped(win, "destroy", G_CALLBACK(destroy_event), m);
    g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(gtk_widget_destroy), win);
    g_signal_connect_swapped(ok, "clicked", G_CALLBACK(ok_verify), m);
    g_signal_connect_swapped(update, "clicked", G_CALLBACK(update_verify), m);
    g_signal_connect_swapped(name, "activate", G_CALLBACK(update_verify), m);
    g_signal_connect_swapped(position, "activate", G_CALLBACK(update_verify), m);
    g_signal_connect(show_hide, "toggled", G_CALLBACK(show_hide_event), m);
  } else {
    g_signal_connect_swapped(win, "destroy", G_CALLBACK(new_destroy_event), m);
    g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(gtk_widget_destroy), win);
    g_signal_connect_swapped(ok, "clicked", G_CALLBACK(new_ok_verify), m);
    g_signal_connect_swapped(name, "activate", G_CALLBACK(new_ok_verify), m);
    g_signal_connect_swapped(position, "activate", G_CALLBACK(new_ok_verify), m);
  }

  gtk_widget_show_all(win);

  m->gui_edit_on = 1;
  m->gui_edit = win;
  m->gui_edit_name = name;
  m->gui_edit_position = position;
}

static mark __new_mark;
void mark_gui_new(void)
{
  if (new_in_use)
    return;

  new_in_use = 1;

  memset(&__new_mark, 0, sizeof(mark));

  mark_gui_edit_or_new(&__new_mark);
}

void mark_gui_new_with_sample(sample)
{
  if (new_in_use)
    return;

  new_in_use = 1;

  memset(&__new_mark, 0, sizeof(mark));
  __new_mark.sample = sample;

  mark_gui_edit_or_new(&__new_mark);
}

void mark_gui_edit(void)
{
  mark_gui_edit_or_new(0);
}

void mark_gui_delete(void)
{
  extern GtkWidget *main_combo_marker_list;
  extern GtkWidget *main_win;
  mark *m = get_mark((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry)));

  if (!m)
    return;

  /* error used as buffer */
  reset_error();
  add_error_char("Are you sure you want to delete the mark '");
  add_error_char((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry)));
  add_error_char("'?\n");
  if (yes_no(main_win, get_error(), 0, 0))
    delete_mark(m);
}

void mark_gui_goto(void)
{
  extern GtkWidget *main_combo_marker_list;
  extern GtkWidget *scrollbar_right_horiz;
  mark *m = get_mark((char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(main_combo_marker_list)->entry)));

  if (!m)
    return;

  gtk_range_set_value(GTK_RANGE(scrollbar_right_horiz), (m->sample-main_mixer->minimum_sample)/4096 - 20);
}

void mark_gui_all_on(void)
{
  int i;

  for (i=0; i<mark_list_size; i++)
    mark_on(mark_list[i]);
}

void mark_gui_all_off(void)
{
  int i;

  for (i=0; i<mark_list_size; i++)
    mark_off(mark_list[i]);
}
