/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "memmap.h"

#include <sys/stat.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "error.h"

typedef struct {
  void *buf;
  dev_t device;
  ino_t inode;
  int count;
} map_list;

map_list *list = 0;
int size = 0;
int maxsize = 0;

/* like mmap, returns -1 if error */
void *memmap(struct stat *buf, int f, char *file)
{
  int i;
  void *b;

  /* search the list */
  for (i=0; i<size; i++)
    if (buf->st_dev == list[i].device && buf->st_ino == list[i].inode) {
      list[i].count++;
      return list[i].buf;
    }

  /* not there */
  if (size == maxsize) {
    map_list *n = realloc(list, (maxsize + 32) * sizeof(map_list));
    if (!n) {
      oom();
      display_error();
      return (void *)-1;
    }
    list = n;
    maxsize += 32;
  }

  b = mmap(0, buf->st_size, PROT_READ, MAP_SHARED, f, 0);

  if (b == (void *)-1) {
    int serr = errno;
    reset_error();
    add_error_char(file);
    add_error_char(":");
    add_error_char(strerror(serr));
    add_error_char("\n");
    display_error();
    return (void *)-1;
  }

  list[size].buf = b;
  list[size].device = buf->st_dev;
  list[size].inode = buf->st_ino;
  list[size].count = 1;
  size++;

  return b;
}

void memunmap(void *b, int s)
{
  int i;

  /* it has to be in there */
  for (i=0; i<size; i++)
    if (list[i].buf == b)
      break;

  if (i == size) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": you should not read this, contact the authors.\n");
    display_error();
    return;
  }

  list[i].count--;

  if (list[i].count)
    return;

  munmap(b, s);

  /* we have to remove it from the list */
  for (; i<size-1; i++)
    list[i] = list[i+1];

  size--;

  /* make some room */
  if (maxsize-size > 32) {
    map_list *new = realloc(list, (maxsize-32) * sizeof(map_list));
    /* if it does not work, no problem */
    if (!new)
      return;
    maxsize -= 32;
    list = new;
  }
}

void cleanup_map(void)
{
  if (size) {
    reset_error();
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_int(__LINE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char(": you should not read this, contact the authors.\n");
    display_error();
  }

  if (!maxsize)
    return;

  free(list);
  list = 0;
  maxsize = 0;
}
