/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "mix_gui.h"

#include <gtk/gtk.h>

#include "error.h"
#include "version.h"

static int gui_on = 0;

static unsigned char overload_left[256];
static unsigned char overload_right[256];
static unsigned char adjust_min_left[256];
static unsigned char adjust_max_left[256];
static unsigned char adjust_min_right[256];
static unsigned char adjust_max_right[256];
static unsigned char adjust_next_point;
static GtkWidget *draw_left;
static GtkWidget *draw_right;
static GtkWidget *awin;

static int draw_expose_event(GtkWidget *draw, GdkEventExpose *ev)
{
  extern GdkGC *red_gc;
  int len = ev->area.width;
  int i;

  if (draw == draw_left)
  for (i=ev->area.x; len; len--, i++)
    gdk_draw_line(ev->window, overload_left[i] ? red_gc : draw->style->black_gc, i, adjust_min_left[i], i, adjust_max_left[i]);

  if (draw == draw_right)
  for (i=ev->area.x; len; len--, i++)
    gdk_draw_line(ev->window, overload_right[i] ? red_gc : draw->style->black_gc, i, adjust_min_right[i], i, adjust_max_right[i]);

  return TRUE;
}

static void adjust_input_destroy(void)
{
  gui_on = 0;
}

void mix_preview_on(void)
{
  GtkWidget *vbox, *hbox;
  GtkWidget *close;
  int i;

  if (gui_on)
    return;

  /* data */
  adjust_next_point = 0;
  for (i=0; i<256; i++) {
    adjust_min_left[i] = adjust_max_left[i] = adjust_min_right[i] = adjust_max_right[i] = 50;
    overload_left[i] = overload_right[i] = 0;
  }

  /* gui */
  awin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width(GTK_CONTAINER(awin), 5);
  /* error used as buffer */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  add_error_char(" - Mix Preview");
  gtk_window_set_title(GTK_WINDOW(awin), get_error());

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(awin), vbox);

  draw_left = gtk_drawing_area_new();
  gtk_widget_modify_bg(draw_left, GTK_STATE_NORMAL, &draw_left->style->white);
  gtk_widget_set_size_request(draw_left, 256, 101);
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), draw_left, TRUE, FALSE, 0);

  draw_right = gtk_drawing_area_new();
  gtk_widget_modify_bg(draw_right, GTK_STATE_NORMAL, &draw_right->style->white);
  gtk_widget_set_size_request(draw_right, 256, 101);
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), draw_right, TRUE, FALSE, 0);

  close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(vbox), close, TRUE, TRUE, 0);

  /* callbacks */
  g_signal_connect(awin, "destroy", G_CALLBACK(adjust_input_destroy), 0);
  g_signal_connect_swapped(close, "clicked", G_CALLBACK(gtk_widget_destroy), awin);
  g_signal_connect(draw_left, "expose-event", G_CALLBACK(draw_expose_event), 0);
  g_signal_connect(draw_right, "expose-event", G_CALLBACK(draw_expose_event), 0);

  gtk_widget_show_all(awin);

  gui_on = 1;
}

void mix_preview_off(void)
{
  if (!gui_on)
    return;

  gtk_widget_destroy(awin);

  gui_on = 0;
}

void mix_preview_update(float *b, int nb)
{
  float min, max;
  int i;
  int overload = 0;

  if (!gui_on)
    return;

  /* if b=0 it means, all points are 0 */
  if (!b) {
    overload_left[adjust_next_point] = 0;
    adjust_min_left[adjust_next_point] = 50;
    adjust_max_left[adjust_next_point] = 50;

    overload_right[adjust_next_point] = 0;
    adjust_min_right[adjust_next_point] = 50;
    adjust_max_right[adjust_next_point] = 50;

    gtk_widget_queue_draw_area(draw_left, adjust_next_point, 0, 1, 101);
    gtk_widget_queue_draw_area(draw_right, adjust_next_point, 0, 1, 101);

    adjust_next_point++;
    return;
  }

  if (adjust_next_point == 0) {
    for (i=0; i<256; i++) {
      adjust_min_left[i] = adjust_max_left[i] = adjust_min_right[i] = adjust_max_right[i] = 50;
      overload_left[i] = overload_right[i] = 0;
    }
    gtk_widget_queue_draw(draw_left);
    gtk_widget_queue_draw(draw_right);
  }

  min = max = b[0];

  for (i=2; i<nb*2; i+=2) {
    if (b[i] < min) min = b[i];
    if (b[i] > max) max = b[i];
  }

  /*  -32768    x     32767
   *    100     y       0
   * y / 100 = (x-32767) / (-32768-32767)
   */
  if (min < -32768) { overload = 1; min = -32768; }
  if (min > 32767) { overload = 1; min = 32767; }
  if (max < -32768) { overload = 1; max = -32768; }
  if (max > 32767) {overload = 1; max = 32767; }
  overload_left[adjust_next_point] = overload;
  adjust_min_left[adjust_next_point] = ((float)min-32767.) * 100. / (-32768.-32767.);
  adjust_max_left[adjust_next_point] = ((float)max-32767.) * 100. / (-32768.-32767.);

  overload = 0;
  min = max = b[1];

  for (i=3; i<nb*2; i+=2) {
    if (b[i] < min) min = b[i];
    if (b[i] > max) max = b[i];
  }

  if (min < -32768) { overload = 1; min = -32768; }
  if (min > 32767) { overload = 1; min = 32767; }
  if (max < -32768) { overload = 1; max = -32768; }
  if (max > 32767) {overload = 1; max = 32767; }
  overload_right[adjust_next_point] = overload;
  adjust_min_right[adjust_next_point] = ((float)min-32767.) * 100. / (-32768.-32767.);
  adjust_max_right[adjust_next_point] = ((float)max-32767.) * 100. / (-32768.-32767.);

  gtk_widget_queue_draw_area(draw_left, adjust_next_point, 0, 1, 101);
  gtk_widget_queue_draw_area(draw_right, adjust_next_point, 0, 1, 101);

  adjust_next_point++;
}

void mix_preview_reset(void)
{
  int i;

  if (!gui_on)
    return;

  adjust_next_point = 0;
  for (i=0; i<256; i++) {
    adjust_min_left[i] = adjust_max_left[i] = adjust_min_right[i] = adjust_max_right[i] = 50;
    overload_left[i] = overload_right[i] = 0;
  }

  gtk_widget_queue_draw(draw_left);
  gtk_widget_queue_draw(draw_right);
}
