/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "pan.h"

#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "pan_gui.h"

static int feed_buffer_short_mono(pan *this, short *buf, int nb_samples, int start)
{
  return this->c.subchannel->c.feed_buffer_short_mono(this->c.subchannel, buf, nb_samples, start);
}

static int feed_buffer_float_mono(pan *this, float *buf, int nb_samples, int start)
{
  return this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, buf, nb_samples, start);
}

static int feed_buffer_short_stereo(pan *this, short *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, buf, nb_samples, start);

  if (this->pan == 0)
    return this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, buf, nb_samples, start);

  if (!this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, buf, nb_samples, start))
    return 0;

  for (i=0; i<nb_samples * 2; i+=2) {
    buf[i] = (float)buf[i] * this->l;
    buf[i+1] = (float)buf[i+1] * this->r;
  }

  return 1;
}

static int feed_buffer_float_stereo(pan *this, float *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start);

  if (this->pan == 0)
    return this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start);

  if (!this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start))
    return 0;

  for (i=0; i<nb_samples * 2; i+=2) {
    buf[i] *= this->l;
    buf[i+1] *= this->r;
  }

  return 1;
}

static int first_point(pan *this)
{
  return this->c.subchannel->c.first_point(this->c.subchannel);
}

static int last_point(pan *this)
{
  return this->c.subchannel->c.last_point(this->c.subchannel);
}

static void reset(pan *this)
{
  this->c.subchannel->c.reset(this->c.subchannel);
}

pan *new_pan(channel *c)
{
  pan *ret = malloc(sizeof(pan));

  if (!ret)
    return 0;

  memset(ret, 0, sizeof(pan));

  ret->c.type = CHANNEL_TYPE_PAN;

  ret->c.subchannel = c;

  ret->c.feed_buffer_short_mono = (int (*)(void *, short *, int, int))feed_buffer_short_mono;
  ret->c.feed_buffer_float_mono = (int (*)(void *, float *, int, int))feed_buffer_float_mono;
  ret->c.feed_buffer_short_stereo = (int (*)(void *, short *, int, int))feed_buffer_short_stereo;
  ret->c.feed_buffer_float_stereo = (int (*)(void *, float *, int, int))feed_buffer_float_stereo;
  ret->c.first_point = (int (*)(void *))first_point;
  ret->c.last_point = (int (*)(void *))last_point;
  ret->c.reset = (void (*)(void *))reset;

  ret->c.open_gui = (void (*)(void *, int))pan_open_gui;
  ret->c.close_gui = (void (*)(void *))pan_close_gui;

  set_pan(ret, 0);

  return ret;
}

void delete_pan(pan *p)
{
  free(p);
}

void set_pan(pan *p, double v)
{
  while (p && p->c.type != CHANNEL_TYPE_PAN)
    p = (pan *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->pan = v;
  p->l = v <= 0 ? 1 : 1 - v;
  p->r = v >= 0 ? 1 : 1 + v;
}
