/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "pos.h"

#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "pos_gui.h"
#include "main_gui.h"

static int feed_buffer_short_mono(pos *this, short *buf, int nb_samples, int start)
{
  short *sub_buf;
  int sub_nb_samples;
  int sub_start;
  int last_point = this->subchannel_start + this->subchannel_nb_samples - 1;
  int buf_start;
  int new_len;
  int s_nb_samples = nb_samples;

  start -= this->global_start;

  buf_start = this->subchannel_start - start;
  if (buf_start >= nb_samples)
    return 0;
  if (buf_start < 0)
    buf_start = 0;
  else
    nb_samples = nb_samples - buf_start;
  if (buf_start+start > last_point)
    return 0;
  new_len = last_point - (start + buf_start) + 1;

  sub_nb_samples = new_len < nb_samples ? new_len : nb_samples;
  sub_buf = (short *)((char *)buf + buf_start * sizeof(short));
  sub_start = start + buf_start;

  memset(buf, 0, sizeof(short) * s_nb_samples);

  return this->c.subchannel->c.feed_buffer_short_mono(this->c.subchannel, sub_buf, sub_nb_samples, sub_start);
}

static int feed_buffer_float_mono(pos *this, float *buf, int nb_samples, int start)
{
  float *sub_buf;
  int sub_nb_samples;
  int sub_start;
  int last_point = this->subchannel_start + this->subchannel_nb_samples - 1;
  int buf_start;
  int new_len;
  int s_nb_samples = nb_samples;

  start -= this->global_start;

  buf_start = this->subchannel_start - start;
  if (buf_start >= nb_samples)
    return 0;
  if (buf_start < 0)
    buf_start = 0;
  else
    nb_samples = nb_samples - buf_start;
  if (buf_start+start > last_point)
    return 0;
  new_len = last_point - (start + buf_start) + 1;

  sub_nb_samples = new_len < nb_samples ? new_len : nb_samples;
  sub_buf = (float *)((char *)buf + buf_start * sizeof(float));
  sub_start = start + buf_start;

  memset(buf, 0, sizeof(float) * s_nb_samples);

  return this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, sub_buf, sub_nb_samples, sub_start);
}

static int feed_buffer_short_stereo(pos *this, short *buf, int nb_samples, int start)
{
  short *sub_buf;
  int sub_nb_samples;
  int sub_start;
  int last_point = this->subchannel_start + this->subchannel_nb_samples - 1;
  int buf_start;
  int new_len;
  int s_nb_samples = nb_samples;

  start -= this->global_start;

  buf_start = this->subchannel_start - start;
  if (buf_start >= nb_samples)
    return 0;
  if (buf_start < 0)
    buf_start = 0;
  else
    nb_samples = nb_samples - buf_start;
  if (buf_start+start > last_point)
    return 0;
  new_len = last_point - (start + buf_start) + 1;

  sub_nb_samples = new_len < nb_samples ? new_len : nb_samples;
  sub_buf = (short *)((char *)buf + buf_start * sizeof(short) * 2);
  sub_start = start + buf_start;

  memset(buf, 0, sizeof(short) * 2 * s_nb_samples);

  return this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, sub_buf, sub_nb_samples, sub_start);
}

static int feed_buffer_float_stereo(pos *this, float *buf, int nb_samples, int start)
{
  float *sub_buf;
  int sub_nb_samples;
  int sub_start;
  int last_point = this->subchannel_start + this->subchannel_nb_samples - 1;
  int buf_start;
  int new_len;
  int s_nb_samples = nb_samples;

  start -= this->global_start;

  buf_start = this->subchannel_start - start;
  if (buf_start >= nb_samples)
    return 0;
  if (buf_start < 0)
    buf_start = 0;
  else
    nb_samples = nb_samples - buf_start;
  if (buf_start+start > last_point)
    return 0;
  new_len = last_point - (start + buf_start) + 1;

  sub_nb_samples = new_len < nb_samples ? new_len : nb_samples;
  sub_buf = (float *)((char *)buf + buf_start * sizeof(float) * 2);
  sub_start = start + buf_start;

  memset(buf, 0, sizeof(float) * 2 * s_nb_samples);

  return this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, sub_buf, sub_nb_samples, sub_start);
}

static int first_point(pos *this)
{
  return this->global_start + this->subchannel_start;
}

static int last_point(pos *this)
{
  return this->global_start + this->subchannel_start + this->subchannel_nb_samples - 1;
}

static void reset(pos *this)
{
  this->c.subchannel->c.reset(this->c.subchannel);
}

pos *new_pos(channel *c)
{
  pos *ret = malloc(sizeof(pos));

  if (!ret)
    return 0;

  memset(ret, 0, sizeof(pos));

  ret->c.type = CHANNEL_TYPE_POS;

  ret->c.subchannel = c;

  ret->c.feed_buffer_short_mono = (int (*)(void *, short *, int, int))feed_buffer_short_mono;
  ret->c.feed_buffer_float_mono = (int (*)(void *, float *, int, int))feed_buffer_float_mono;
  ret->c.feed_buffer_short_stereo = (int (*)(void *, short *, int, int))feed_buffer_short_stereo;
  ret->c.feed_buffer_float_stereo = (int (*)(void *, float *, int, int))feed_buffer_float_stereo;
  ret->c.first_point = (int (*)(void *))first_point;
  ret->c.last_point = (int (*)(void *))last_point;
  ret->c.reset = (void (*)(void *))reset;

  ret->c.open_gui = (void (*)(void *, int))pos_open_gui;
  ret->c.close_gui = (void (*)(void *))pos_close_gui;

  set_global_start(ret, 0);
  set_subchannel_start(ret, 0);
  set_subchannel_nb_samples(ret, c->c.last_point(c)+1);

  return ret;
}

void delete_pos(pos *v)
{
  free(v);
}

void set_global_start(pos *v, int s)
{
  while (v && v->c.type != CHANNEL_TYPE_POS)
    v = (pos *)v->c.subchannel;

  if (!v) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  v->global_start = s;

  main_gui_update_track_view();

  modify = 1;
}

void set_subchannel_start(pos *v, int s)
{
  int maxs;

  while (v && v->c.type != CHANNEL_TYPE_POS)
    v = (pos *)v->c.subchannel;

  if (!v) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  maxs = v->c.subchannel->c.last_point(v->c.subchannel);

  if (s < 0) s = 0;
  if (s > maxs) s = maxs;

  v->subchannel_start = s;

  main_gui_redraw_track_view();

  modify = 1;
}

void set_subchannel_nb_samples(pos *v, int s)
{
  /* here, we heavilly suppose the the last_point of subchannel indicates its length
   * meaning we suppose the subchannel is an input channel.
   */
  int maxs;

  while (v && v->c.type != CHANNEL_TYPE_POS)
    v = (pos *)v->c.subchannel;

  if (!v) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  maxs = v->c.subchannel->c.last_point(v->c.subchannel) - v->subchannel_start + 1;

  if (s < 1) s = 1;
  if (s > maxs) s = maxs;

  v->subchannel_nb_samples = s;

  main_gui_redraw_track_view();

  modify = 1;
}
