/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "pos_gui.h"

#include <gtk/gtk.h>
#include <stdlib.h>
#include <errno.h>

#include "pos.h"
#include "effects_gui.h"
#include "error.h"

static void destroy_event(pos *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  effects_gui_force_redraw();
}

static void entry_event(pos *p)
{
  long val1, val2, val3;
  const char *c1, *c2, *c3;
  char *end;

  if (!p->c.gui_on)
    return;

  /* let's check that entries are ok, put the old value if not */
  c1 = gtk_entry_get_text(p->entry1);
  c2 = gtk_entry_get_text(p->entry2);
  c3 = gtk_entry_get_text(p->entry3);

  val1 = strtol(c1, &end, 10);
  if (*c1 == 0 || *end != 0)
    val1 = p->global_start;

  val2 = strtol(c2, &end, 10);
  if (errno == ERANGE) {
    if (val2 == LONG_MIN)
      val2 = 0;
    else if (val2 == LONG_MAX)
      val2 = p->c.subchannel->c.last_point(p->c.subchannel);
  }
  if (*c1 == 0 || *end != 0)
    val2 = p->subchannel_start;

  val3 = strtol(c3, &end, 10);
  if (errno == ERANGE) {
    if (val3 == LONG_MIN)
      val3 = 1;
    else if (val3 == LONG_MAX)
      val3 = p->c.subchannel->c.last_point(p->c.subchannel) - val2 + 1;
  } else
    val3 = val3 - val2 + 1;
  if (*c1 == 0 || *end != 0)
    val3 = p->subchannel_nb_samples;

  /* now, do the change */
  set_global_start(p, val1);
  set_subchannel_start(p, val2);
  set_subchannel_nb_samples(p, val3);

  /* and update */
  update_entry_fields(p);
}

void pos_open_gui(pos *p, int track_number)
{
  GtkWidget *win;
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *entry1, *entry2, *entry3;
  GtkWidget *label1, *label2, *label3;

  if (p->c.gui_on)
    return;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width(GTK_CONTAINER(win), 5);

  box = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(win), box);

  /* set track number */
  /* user error as buffer */
  reset_error();
  add_error_char("# ");
  add_error_int(track_number+1);
  label = gtk_label_new(get_error());

  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), FALSE, FALSE, 0);

  /* error used as buffer */
  reset_error();
  add_error_int(p->global_start);
  label1 = gtk_label_new(get_error());
  reset_error();
  add_error_int(p->subchannel_start);
  label2 = gtk_label_new(get_error());
  reset_error();
  add_error_int(p->subchannel_start+p->subchannel_nb_samples-1);
  label3 = gtk_label_new(get_error());

  hbox = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Starting point"), FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), label1, FALSE, FALSE, 0);
  entry1 = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(box), entry1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("First audio sample"), FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
  entry2 = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(box), entry2, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Last audio sample"), FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
  entry3 = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(box), entry3, FALSE, FALSE, 0);

  /* error used as buffer */
  reset_error();
  add_error_int(p->global_start);
  gtk_entry_set_text(GTK_ENTRY(entry1), get_error());
  reset_error();
  add_error_int(p->subchannel_start);
  gtk_entry_set_text(GTK_ENTRY(entry2), get_error());
  reset_error();
  add_error_int(p->subchannel_start+p->subchannel_nb_samples-1);
  gtk_entry_set_text(GTK_ENTRY(entry3), get_error());

  /* error used as buffer */
  reset_error();
  add_error_char("(max ");
  add_error_int(p->c.subchannel->c.last_point(p->c.subchannel));
  add_error_char(")");
  gtk_box_pack_start(GTK_BOX(box), gtk_label_new(get_error()), FALSE, FALSE, 0);

  /* callbacks */
  g_signal_connect_swapped(entry1, "activate", G_CALLBACK(entry_event), p);
  g_signal_connect_swapped(entry2, "activate", G_CALLBACK(entry_event), p);
  g_signal_connect_swapped(entry3, "activate", G_CALLBACK(entry_event), p);

  g_signal_connect_swapped(win, "destroy", G_CALLBACK(destroy_event), p);

  /* save stuff */
  p->c.label = label;
  p->win = win;
  p->entry1 = entry1;
  p->entry2 = entry2;
  p->entry3 = entry3;
  p->label1 = label1;
  p->label2 = label2;
  p->label3 = label3;

  gtk_widget_show_all(win);

  p->c.gui_on = 1;
}

void pos_close_gui(pos *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  gtk_widget_destroy(p->win);
}

void update_entry_fields(pos *p)
{
  if (!p->c.gui_on)
    return;

  /* error used as buffer */
  reset_error();
  add_error_int(p->global_start);
  gtk_entry_set_text(GTK_ENTRY(p->entry1), get_error());
  gtk_label_set_text(GTK_LABEL(p->label1), get_error());
  reset_error();
  add_error_int(p->subchannel_start);
  gtk_entry_set_text(GTK_ENTRY(p->entry2), get_error());
  gtk_label_set_text(GTK_LABEL(p->label2), get_error());
  reset_error();
  add_error_int(p->subchannel_start + p->subchannel_nb_samples - 1);
  gtk_entry_set_text(GTK_ENTRY(p->entry3), get_error());
  gtk_label_set_text(GTK_LABEL(p->label3), get_error());
}
