/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

/*
 * this code is ripped and adapted from sc1_1425 coming from
 * swh-plugins-0.4.11.tar.gz (see http://plugin.org.uk/)
 */

#include "sc1.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "error.h"
#include "sc1_gui.h"

/* ladspa-util.h stuff */

typedef union {
        float f;
        int i;
} ls_pcast32;

static inline int f_round(float f)
{
        ls_pcast32 p;

        p.f = f;
        p.f += (3<<22);

        return p.i - 0x4b400000;
}

/* db.h and db.c stuff */

#define DB_TABLE_SIZE 1024
#define DB_MIN -60.0f
#define DB_MAX 24.0f
#define LIN_TABLE_SIZE 1024
#define LIN_MIN 0.0000000002f
#define LIN_MAX 9.0f

#define db2lin(a) f_db2lin_lerp(a)
#define lin2db(a) f_lin2db_lerp(a)

float db_data[DB_TABLE_SIZE];
float lin_data[LIN_TABLE_SIZE];

static inline float f_db2lin_lerp(float db)
{
        float scale = (db - DB_MIN) * (float)LIN_TABLE_SIZE / (DB_MAX - DB_MIN);
        int base = f_round(scale - 0.5f);
        float ofs = scale - base;

        if (base < 1) {
                return 0.0f;
        } else if (base > LIN_TABLE_SIZE - 3) {
                return lin_data[LIN_TABLE_SIZE - 2];
        }
        return (1.0f - ofs) * lin_data[base] + ofs * lin_data[base+1];
}

static inline float f_lin2db_lerp(float lin)
{
        float scale = (lin - LIN_MIN) * (float)DB_TABLE_SIZE / (LIN_MAX - LIN_MIN);
        int base = f_round(scale - 0.5f);
        float ofs = scale - base;

        if (base < 2) {
                return db_data[2] * scale * 0.5f - 23.0f * (2.0f - scale);
        } else if (base > DB_TABLE_SIZE - 2) {
                return db_data[DB_TABLE_SIZE - 1];
        }
        return (1.0f - ofs) * db_data[base] + ofs * db_data[base+1];
}

static int inited = 0;
void db_init()
{
        unsigned int i;

        if (inited)
          return;
        inited = 1;

        for (i=0; i<LIN_TABLE_SIZE; i++) {
                lin_data[i] = powf(10.0f, ((DB_MAX - DB_MIN) *
                        (float)i/(float)LIN_TABLE_SIZE + DB_MIN) / 20.0f);
        }

        for (i=0; i<DB_TABLE_SIZE; i++) {
                db_data[i] = 20.0f * log10f((LIN_MAX - LIN_MIN) *
                        (float)i/(float)DB_TABLE_SIZE + LIN_MIN);
        }
}

/* rms stuff */

inline static float rms_env_process(rms_env *r, const float x)
{
        r->sum -= r->buffer[r->pos];
        r->sum += x;
        r->buffer[r->pos] = x;
        r->pos = (r->pos + 1) & (RMS_BUF_SIZE - 1);

        /* don't ask me why, I got sum<0 sometimes (round errors ?) [Sed, dec 12 2004]*/
        if (r->sum <0) r->sum = 0;

        return sqrt(r->sum / (float)RMS_BUF_SIZE);
}

void rms_env_reset(rms_env *r)
{
        unsigned int i;

        for (i=0; i<RMS_BUF_SIZE; i++) {
                r->buffer[i] = 0.0f;
        }
        r->pos = 0;
        r->sum = 0.0f;
}

/* how to handle the input buffer of sc1_run */
#define MONO		0
#define STEREO_LEFT	1
#define STEREO_RIGHT	2

void sc1_run(sc1 *sc, sc1_data *d, float *buf, int nb_samples, int type)
{
  int sample_count = nb_samples;

        /* Attack time (ms) (float value) */
        const float attack = sc->attack;

        /* Release time (ms) (float value) */
        const float release = sc->release;

        /* Threshold level (dB) (float value) */
        const float threshold = sc->threshold;

        /* Ratio (1:n) (float value) */
        const float ratio = sc->ratio;

        /* Knee radius (dB) (float value) */
        const float knee = sc->knee;

        /* Makeup gain (dB) (float value) */
        const float makeup_gain = sc->makeup_gain;

        /* Input (array of floats of length sample_count) */
        const float * const input = buf;

        /* Output (array of floats of length sample_count) */
        float * const output = buf;
        float amp = d->amp;
        float * as = d->as;
        unsigned int count = d->count;
        float env = d->env;
        float gain = d->gain;
        float gain_t = d->gain_t;
        rms_env * rms = &d->rms;
        float sum = d->sum;

      unsigned long pos;

      const float ga = as[f_round(attack * 0.001f * (float)(A_TBL-1))];
      const float gr = as[f_round(release * 0.001f * (float)(A_TBL-1))];
      const float rs = (ratio - 1.0f) / ratio;
      const float mug = db2lin(makeup_gain);
      const float knee_min = db2lin(threshold - knee);
      const float knee_max = db2lin(threshold + knee);
      const float ef_a = ga * 0.25f;
      const float ef_ai = 1.0f - ef_a;

      for (pos = 0; pos < sample_count; pos++) {
        register float in = type == MONO ? input[pos] : type == STEREO_LEFT ? input[pos*2] : input[pos*2+1];
        in /= 32768;
        sum += in * in;

        if (amp > env) {
          env = env * ga + amp * (1.0f - ga);
        } else {
          env = env * gr + amp * (1.0f - gr);
        }
        if (count++ % 4 == 3) {
          amp = rms_env_process(rms, sum * 0.25f);
          sum = 0.0f;
          if (env <= knee_min) {
            gain_t = 1.0f;
          } else if (env < knee_max) {
            const float x = -(threshold - knee - lin2db(env)) / knee;
            gain_t = db2lin(-knee * rs * x * x * 0.25f);
          } else {
            gain_t = db2lin((threshold - lin2db(env)) * rs);
          }
        }
        gain = gain * ef_a + gain_t * ef_ai;
        if (type == MONO)
          output[pos] = input[pos] * gain * mug;
        else if (type == STEREO_LEFT)
          output[pos*2] = input[pos*2] * gain * mug;
        else
          output[pos*2+1] = input[pos*2+1] * gain * mug;
      }
      d->sum = sum;
      d->amp = amp;
      d->gain = gain;
      d->gain_t = gain_t;
      d->env = env;
      d->count = count;
}

void sc1_reset(sc1_data *d)
{
  int i;

  /* it's bad to hardcode stuff like this, I know... */
  d->sample_rate = 44100;

  rms_env_reset(&d->rms);

  d->sum = 0.0f;
  d->amp = 0.0f;
  d->gain = 0.0f;
  d->gain_t = 0.0f;
  d->env = 0.0f;
  d->count = 0;

  d->as[0] = 1.0f;
  for (i=1; i<A_TBL; i++)
    d->as[i] = expf(-1.0f / (d->sample_rate * (float)i / (float)A_TBL));
}

static int short_error_shown = 0;

static int feed_buffer_short_mono(sc1 *this, short *buf, int nb_samples, int start)
{
  if (short_error_shown)
    return 0;

  short_error_shown = 1;

  reset_error();
  add_error_char("You should not read this.\n");
  add_error_char("feeding a mix buffer in short mode should not be used.\n");
  add_error_char(__FILE__);
  add_error_char(":");
  add_error_char(__FUNCTION__);
  add_error_char("\n");
  display_error();

  return 0;
}

static int feed_buffer_float_mono(sc1 *this, float *buf, int nb_samples, int start)
{
  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, buf, nb_samples, start);

  /* not sure if it's correct, shouldn't we reset stuff here ? */
  if (!this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, buf, nb_samples, start))
    return 0;

  sc1_run(this, &this->mono_data, buf, nb_samples, MONO);

  return 1;
}

static int feed_buffer_short_stereo(sc1 *this, short *buf, int nb_samples, int start)
{
  if (short_error_shown)
    return 0;

  short_error_shown = 1;

  reset_error();
  add_error_char("You should not read this.\n");
  add_error_char("feeding a mix buffer in short mode should not be used.\n");
  add_error_char(__FILE__);
  add_error_char(":");
  add_error_char(__FUNCTION__);
  add_error_char("\n");
  display_error();

  return 0;
}

static int feed_buffer_float_stereo(sc1 *this, float *buf, int nb_samples, int start)
{
  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start);

  /* not sure if it's correct, shouldn't we reset stuff here ? */
  if (!this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start))
    return 0;

  sc1_run(this, &this->left_data, buf, nb_samples, STEREO_LEFT);
  sc1_run(this, &this->right_data, buf, nb_samples, STEREO_RIGHT);

  return 1;
}

static int first_point(sc1 *this)
{
  return this->c.subchannel->c.first_point(this->c.subchannel);
}

static int last_point(sc1 *this)
{
  return this->c.subchannel->c.last_point(this->c.subchannel);
}

static void reset(sc1 *this)
{
  this->c.subchannel->c.reset(this->c.subchannel);

  short_error_shown = 0;

  sc1_reset(&this->mono_data);
  sc1_reset(&this->left_data);
  sc1_reset(&this->right_data);
}

sc1 *new_sc1(channel *c)
{
  sc1 *ret= malloc(sizeof(sc1));

  if (!ret)
    return 0;

  memset(ret, 0, sizeof(sc1));

  ret->c.type = CHANNEL_TYPE_SC1;

  ret->c.subchannel = c;

  ret->c.feed_buffer_short_mono = (int (*)(void *, short *, int, int))feed_buffer_short_mono;
  ret->c.feed_buffer_float_mono = (int (*)(void *, float *, int, int))feed_buffer_float_mono;
  ret->c.feed_buffer_short_stereo = (int (*)(void *, short *, int, int))feed_buffer_short_stereo;
  ret->c.feed_buffer_float_stereo = (int (*)(void *, float *, int, int))feed_buffer_float_stereo;
  ret->c.first_point = (int (*)(void *))first_point;
  ret->c.last_point = (int (*)(void *))last_point;
  ret->c.reset = (void (*)(void *))reset;

  ret->c.open_gui = (void (*)(void *, int))sc1_open_gui;
  ret->c.close_gui = (void (*)(void *))sc1_close_gui;

  set_attack(ret, 2);
  set_release(ret, 400);
  set_threshold(ret, 0);
  set_ratio(ret, 1);
  set_knee(ret, 1);
  set_makeup_gain(ret, 0);

  db_init();

  sc1_reset(&ret->mono_data);
  sc1_reset(&ret->left_data);
  sc1_reset(&ret->right_data);

  return ret;
}

void delete_sc1(sc1 *s)
{
  free(s);
}

void set_attack(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->attack = a;
}

void set_release(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->release = a;
}

void set_threshold(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->threshold = a;
}

void set_ratio(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->ratio = a;
}

void set_knee(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->knee = a;
}

void set_makeup_gain(sc1 *p, double a)
{
  while (p && p->c.type != CHANNEL_TYPE_SC1)
    p = (sc1 *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->makeup_gain = a;
}
