/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#ifndef _SC1_H_
#define _SC1_H_

#include "channel.h"

#define RMS_BUF_SIZE 64

typedef struct {
        float        buffer[RMS_BUF_SIZE];
        unsigned int pos;
        float        sum;
} rms_env;

#define A_TBL	256

typedef struct {
  rms_env rms;
  float sum;
  float amp;
  float gain;
  float gain_t;
  float env;
  float count;
  float sample_rate;
  float as[A_TBL];
} sc1_data;

typedef struct {
  channel_common c;
  float attack;		/* 2ms .. 400ms */	/* The attack time in milliseconds. */
  float release;	/* 2ms .. 800ms */	/* The release time in milliseconds. */
  float threshold;	/* -30dB .. 0dB */	/* The point at which the compressor will start to kick in. */
  float ratio;		/* 1 .. 10 */		/* The gain reduction ratio used when the signal level exceeds the threshold. */
  float knee;		/* 1dB .. 10dB */	/* The distance from the threshold where the knee curve starts. */
  float makeup_gain;	/* 0dB .. 24dB */	/* Controls the gain of the makeup input signal in dB's. */
  sc1_data mono_data;
  sc1_data left_data;
  sc1_data right_data;
  /* gui stuff */
  void *win;
  void *attack_scale;
  void *release_scale;
  void *threshold_scale;
  void *ratio_scale;
  void *knee_scale;
  void *makeup_gain_scale;
} sc1;

sc1 *new_sc1(channel *c);
void delete_sc1(sc1 *);

void set_attack(sc1 *, double);
void set_release(sc1 *, double);
void set_threshold(sc1 *, double);
void set_ratio(sc1 *, double);
void set_knee(sc1 *, double);
void set_makeup_gain(sc1 *, double);

#endif /* _SC1_H_ */
