/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "sc1_gui.h"

#include <gtk/gtk.h>

#include "sc1.h"
#include "error.h"
#include "main_gui.h"
#include "effects_gui.h"

static void attack_scale_change_event(sc1 *p)
{
  modify = 1;

  set_attack(p, gtk_range_get_value(GTK_RANGE(p->attack_scale)));
}

static void release_scale_change_event(sc1 *p)
{
  modify = 1;

  set_release(p, gtk_range_get_value(GTK_RANGE(p->release_scale)));
}

static void threshold_scale_change_event(sc1 *p)
{
  modify = 1;

  set_threshold(p, gtk_range_get_value(GTK_RANGE(p->threshold_scale)));
}

static void ratio_scale_change_event(sc1 *p)
{
  modify = 1;

  set_ratio(p, gtk_range_get_value(GTK_RANGE(p->ratio_scale)));
}

static void knee_scale_change_event(sc1 *p)
{
  modify = 1;

  set_knee(p, gtk_range_get_value(GTK_RANGE(p->knee_scale)));
}

static void makeup_gain_scale_change_event(sc1 *p)
{
  modify = 1;

  set_makeup_gain(p, gtk_range_get_value(GTK_RANGE(p->makeup_gain_scale)));
}

static void destroy_event(sc1 *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  effects_gui_force_redraw();
}

void sc1_open_gui(sc1 *p, int track_number)
{
  GtkWidget *win;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *attack_scale;
  GtkWidget *release_scale;
  GtkWidget *threshold_scale;
  GtkWidget *ratio_scale;
  GtkWidget *knee_scale;
  GtkWidget *makeup_gain_scale;

  if (p->c.gui_on)
    return;

  /* create gui */
  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  /* set track number */
  /* use error as buffer */
  reset_error();
  add_error_char("# ");
  add_error_int(track_number+1);
  label = gtk_label_new(get_error());

  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

  /* attack */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Attack (ms)"), FALSE, FALSE, 0);
  attack_scale = gtk_hscale_new_with_range(2, 400, 1);
  gtk_range_set_increments(GTK_RANGE(attack_scale), 1, 10);
  gtk_range_set_value(GTK_RANGE(attack_scale), p->attack);
  gtk_widget_set_size_request(attack_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(attack_scale), 0);
  gtk_scale_set_value_pos(GTK_SCALE(attack_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(attack_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), attack_scale, TRUE, TRUE, 0);

  /* release */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Release (ms)"), FALSE, FALSE, 0);
  release_scale = gtk_hscale_new_with_range(2, 800, 1);
  gtk_range_set_increments(GTK_RANGE(release_scale), 1, 20);
  gtk_range_set_value(GTK_RANGE(release_scale), p->release);
  gtk_widget_set_size_request(release_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(release_scale), 0);
  gtk_scale_set_value_pos(GTK_SCALE(release_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(release_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), release_scale, TRUE, TRUE, 0);

  /* threshold */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Threshold (dB)"), FALSE, FALSE, 0);
  threshold_scale = gtk_hscale_new_with_range(-30, 0, 0.01);
  gtk_range_set_increments(GTK_RANGE(threshold_scale), 0.1, 1);
  gtk_range_set_value(GTK_RANGE(threshold_scale), p->threshold);
  gtk_widget_set_size_request(threshold_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(threshold_scale), 2);
  gtk_scale_set_value_pos(GTK_SCALE(threshold_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(threshold_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), threshold_scale, TRUE, TRUE, 0);

  /* ratio */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Ratio (1:n)"), FALSE, FALSE, 0);
  ratio_scale = gtk_hscale_new_with_range(1, 10, 0.01);
  gtk_range_set_increments(GTK_RANGE(ratio_scale), 0.1, 1);
  gtk_range_set_value(GTK_RANGE(ratio_scale), p->ratio);
  gtk_widget_set_size_request(ratio_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(ratio_scale), 2);
  gtk_scale_set_value_pos(GTK_SCALE(ratio_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(ratio_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), ratio_scale, TRUE, TRUE, 0);

  /* knee */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Knee (dB)"), FALSE, FALSE, 0);
  knee_scale = gtk_hscale_new_with_range(1, 10, 0.01);
  gtk_range_set_increments(GTK_RANGE(knee_scale), 0.1, 1);
  gtk_range_set_value(GTK_RANGE(knee_scale), p->knee);
  gtk_widget_set_size_request(knee_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(knee_scale), 2);
  gtk_scale_set_value_pos(GTK_SCALE(knee_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(knee_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), knee_scale, TRUE, TRUE, 0);

  /* makeup_gain */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Makeup Gain (dB)"), FALSE, FALSE, 0);
  makeup_gain_scale = gtk_hscale_new_with_range(0, 24, 0.01);
  gtk_range_set_increments(GTK_RANGE(makeup_gain_scale), 0.1, 1);
  gtk_range_set_value(GTK_RANGE(makeup_gain_scale), p->makeup_gain);
  gtk_widget_set_size_request(makeup_gain_scale, 200, 30);
  gtk_scale_set_digits(GTK_SCALE(makeup_gain_scale), 2);
  gtk_scale_set_value_pos(GTK_SCALE(makeup_gain_scale), GTK_POS_LEFT);
  gtk_range_set_update_policy(GTK_RANGE(makeup_gain_scale), GTK_UPDATE_CONTINUOUS);
  gtk_box_pack_start(GTK_BOX(hbox), makeup_gain_scale, TRUE, TRUE, 0);

  /* callbacks */
  g_signal_connect_swapped(attack_scale, "value-changed", G_CALLBACK(attack_scale_change_event), p);
  g_signal_connect_swapped(release_scale, "value-changed", G_CALLBACK(release_scale_change_event), p);
  g_signal_connect_swapped(threshold_scale, "value-changed", G_CALLBACK(threshold_scale_change_event), p);
  g_signal_connect_swapped(ratio_scale, "value-changed", G_CALLBACK(ratio_scale_change_event), p);
  g_signal_connect_swapped(knee_scale, "value-changed", G_CALLBACK(knee_scale_change_event), p);
  g_signal_connect_swapped(makeup_gain_scale, "value-changed", G_CALLBACK(makeup_gain_scale_change_event), p);
  g_signal_connect_swapped(win, "destroy", G_CALLBACK(destroy_event), p);

  gtk_widget_show_all(win);

  /* done, let's save data stuff */
  p->c.gui_on = 1;
  p->win = win;
  p->attack_scale = attack_scale;
  p->release_scale = release_scale;
  p->threshold_scale = threshold_scale;
  p->ratio_scale = ratio_scale;
  p->knee_scale = knee_scale;
  p->makeup_gain_scale = makeup_gain_scale;
  p->c.label = label;
}

void sc1_close_gui(sc1 *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  gtk_widget_destroy(p->win);
}
