/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "setup_gui.h"

#include <gtk/gtk.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "error.h"
#include "version.h"

static int gui_on = 0;

static GtkWidget *win;
static GtkWidget *audio_entry;

static char _audio_file[FILENAME_MAX+1] = "/dev/dsp";
char *audio_file = _audio_file;

/* return 0 if no ok, 1 if ok */
static int try_apply(void)
{
  struct stat buf;
  char *c = (char *)gtk_entry_get_text(GTK_ENTRY(audio_entry));

  if (strlen(c) == 0) {
    reset_error();
    add_error_char("You must provide an audio device.\n(Try \"/dev/dsp\".)\n");
    display_error();
    return 0;
  }

  if (strlen(c) > FILENAME_MAX) {
    reset_error();
    add_error_char(c);
    add_error_char(": file name too long (");
    add_error_int(FILENAME_MAX);
    add_error_char(" is the maximum).\n(Try \"/dev/dsp\".)\n");
    display_error();
    return 0;
  }

  if (stat(c, &buf) == -1) {
    int serr = errno;
    reset_error();
    add_error_char(c);
    add_error_char(": ");
    add_error_char(strerror(serr));
    add_error_char("\n(Try \"/dev/dsp\".)\n");
    display_error();
    return 0;
  }

  /* a basic check, so we don't destroy any regular files. */
  if (!S_ISCHR(buf.st_mode)) {
    reset_error();
    add_error_char(c);
    add_error_char(": this file is not an audio device.\n");
    add_error_char("(Try \"/dev/dsp\".)\n");
    display_error();
    return 0;
  }

  strcpy(audio_file, c);

  return 1;
}

static void apply_event(void)
{
  try_apply();
}

static void ok_event(void)
{
  if (!try_apply())
    return;

  gtk_widget_destroy(win);
}

static void destroy_event(void)
{
  gui_on = 0;
}

void setup(void)
{
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *apply, *ok, *cancel;

  if (gui_on)
    return;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width(GTK_CONTAINER(win), 5);
  /* error used as buffer */
  reset_error();
  add_error_char("Songs ");
  add_error_char(VERSION);
  add_error_char(": Preferences");
  gtk_window_set_title(GTK_WINDOW(win), get_error());

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(win), vbox);

  /* audio device */
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Audio device"), FALSE, FALSE, 0);
  audio_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), audio_entry, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(audio_entry), audio_file);

  /* apply - ok - cancel */
  hbox = gtk_hbox_new(TRUE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  apply = gtk_button_new_with_mnemonic("_Apply");
  ok = gtk_button_new_with_mnemonic("_OK");
  cancel = gtk_button_new_with_mnemonic("_Cancel");
  gtk_box_pack_end(GTK_BOX(hbox), ok, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(hbox), cancel, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(hbox), apply, FALSE, FALSE, 0);

  /* callbacks */
  g_signal_connect(apply, "clicked", G_CALLBACK(apply_event), 0);
  g_signal_connect(ok, "clicked", G_CALLBACK(ok_event), 0);
  g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(gtk_widget_destroy), win);
  g_signal_connect(win, "destroy", G_CALLBACK(destroy_event), 0);

  gtk_widget_show_all(win);

  gui_on = 1;
}
