/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "toolbox_gui.h"

#include <gtk/gtk.h>

#include "track_gui.h"

static GtkWidget *toolbox_marker;
static GtkWidget *toolbox_move_track;
static GtkWidget *toolbox_left_track;
static GtkWidget *toolbox_right_track;

#define CURRENT_MARKER	0
#define CURRENT_MOVE	1
#define CURRENT_LEFT	2
#define CURRENT_RIGHT	3

static int current = CURRENT_MARKER;

static void marker_toggled_event(void)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toolbox_marker)) == TRUE)
    current = CURRENT_MARKER;
}

static void move_track_toggled_event(void)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toolbox_move_track)) == TRUE)
    current = CURRENT_MOVE;
}

static void left_track_toggled_event(void)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toolbox_left_track)) == TRUE)
    current = CURRENT_LEFT;
}

static void right_track_toggled_event(void)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toolbox_right_track)) == TRUE)
    current = CURRENT_RIGHT;
}

void *create_toolbox(void)
{
  GtkWidget *toolbox;

  toolbox = gtk_hbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(toolbox), 5);
  
  toolbox_marker = gtk_radio_button_new_with_label(0, "Mark");
  toolbox_move_track = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(toolbox_marker), "_Move");
  toolbox_left_track = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(toolbox_move_track), "_Left");
  toolbox_right_track = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(toolbox_left_track), "Ri_ght");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toolbox_marker), TRUE);

  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(toolbox_marker), FALSE);
  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(toolbox_move_track), FALSE);
  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(toolbox_left_track), FALSE);
  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(toolbox_right_track), FALSE);
  
  widget_current_point = gtk_label_new("");
  
  gtk_box_pack_start(GTK_BOX(toolbox), toolbox_marker, FALSE, FALSE, 0); 
  gtk_box_pack_start(GTK_BOX(toolbox), toolbox_move_track, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toolbox), toolbox_left_track, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toolbox), toolbox_right_track, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(toolbox), widget_current_point, FALSE, FALSE, 0);

  g_signal_connect(toolbox_marker, "toggled", G_CALLBACK(marker_toggled_event), 0);
  g_signal_connect(toolbox_move_track, "toggled", G_CALLBACK(move_track_toggled_event), 0);
  g_signal_connect(toolbox_left_track, "toggled", G_CALLBACK(left_track_toggled_event), 0);
  g_signal_connect(toolbox_right_track, "toggled", G_CALLBACK(right_track_toggled_event), 0);

  return toolbox;
}

int toolbox_mark(void)
{
  return current == CURRENT_MARKER;
}

int toolbox_move(void)
{
  return current == CURRENT_MOVE;
}

int toolbox_left(void)
{
  return current == CURRENT_LEFT;
}

int toolbox_right(void)
{
  return current == CURRENT_RIGHT;
}
