/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "track_gui.h"

#include <gtk/gtk.h>
#include <stdio.h>

#include "track.h"
#include "error.h"
#include "main_gui.h"
#include "effects_gui.h"
#include "yes_no_gui.h"
#include "mixer.h"
#include "pos.h"
#include "pos_gui.h"
#include "toolbox_gui.h"

static void effects_event(track *t)
{
  effects_gui_show_track(t);
}

static void mute_event(track *t)
{
  t->mute = !!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t->mute_button));

  modify = 1;
}

static track *to_be_deleted;
static void do_delete(void)
{
  delete_track(to_be_deleted);

  modify = 1;
}

static void delete_track_event(track *t)
{
  extern void *main_win;

  to_be_deleted = t;

  yes_no(main_win, "Are you sure you want to delete this track?\n", do_delete, 0);
}

void track_gui(track *t)
{
  GtkWidget *box = gtk_vbox_new(FALSE, 10);
  GtkWidget *l;
  GtkWidget *e = gtk_entry_new();
  GtkWidget *hbox;
  GtkWidget *mute;
  GtkWidget *effects;
  GtkWidget *delete_track;
  GtkWidget *sep;

  /* track number and track name zone */
  /* here error stuff is used as a buffer */
  reset_error();
  add_error_char("# ");
  add_error_int(t->track_number+1);
  l = gtk_label_new(get_error());

  gtk_entry_set_width_chars(GTK_ENTRY(e), 20);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), e, TRUE, TRUE, 0);

  /* second line */
  hbox = gtk_hbox_new(FALSE, 0);

  mute = gtk_check_button_new_with_label("Mute");
  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(mute), TRUE);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mute), t->mute);
  effects = gtk_button_new_with_label("Effects");
  delete_track = gtk_button_new_with_label("Delete");
  gtk_box_pack_start(GTK_BOX(hbox), mute, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), effects, TRUE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), delete_track, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);

  /* separator */
  sep = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(box), sep, FALSE, FALSE, 5);

  /* callbacks */
  g_signal_connect_swapped(mute, "toggled", G_CALLBACK(mute_event), t);
  g_signal_connect_swapped(effects, "clicked", G_CALLBACK(effects_event), t);
  g_signal_connect_swapped(delete_track, "clicked", G_CALLBACK(delete_track_event), t);

  /* done */
  gtk_widget_show_all(box);

  /* memorize */
  t->box = box;
  t->name_entry = e;
  t->label = l;
  t->mute_button = mute;

  t->gui_on = 1;
}

void track_ungui(track *t)
{
  channel *c = t->channel;

  if (t->gui_on)
    gtk_widget_destroy(t->box);
  t->gui_on = 0;

  while (c) {
    if (c->c.close_gui)
      c->c.close_gui(c);

    c = c->c.subchannel;
  }
}

char *get_track_name(track *t)
{
  if (!t->gui_on)
    return "error: you should not read this, contact the authors";

  return (char *)gtk_entry_get_text(GTK_ENTRY(t->name_entry));
}

void set_track_name(track *t, char *name)
{
  if (!t->gui_on)
    return;

  gtk_entry_set_text(GTK_ENTRY(t->name_entry), name);

  modify = 1;
}

/* this is a label supposed to display the sample below the pointer
 * there must be bugs around the handling code (unsupported cases,
 * bad cases), the display is not always correct, but hey, it's
 * mostly an informational stuff, not very useful.
 */
void *widget_current_point;

static void update_current_point(track *t, double x)
{
  extern GtkWidget *track_list_right;
  char zo[64];
  int sample;

  if (t != (track *)track_list_right) {
    channel *c = t->channel;
    while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
    x += (((pos *)c)->global_start-main_mixer->minimum_sample+4095)/4096;
  }

  sample = x * 4096 + main_mixer->minimum_sample;

  sprintf(zo, "%d", sample);

  gtk_label_set_text(GTK_LABEL(widget_current_point), zo);
}

static int is_moving = 0;
static int last_position;
static track *moving_track;

int track_pointer_motion_event(void *widget, void *ev)
{
  if (is_moving) {
    if (toolbox_move()) {
      channel *c = moving_track->channel;
      int diff = ((GdkEventMotion *)ev)->x_root - last_position;
      diff *= 4096;
      while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
      set_global_start((pos *)c, ((pos *)c)->global_start + diff);
      update_entry_fields((pos *)c);
      last_position = ((GdkEventMotion *)ev)->x_root;
    }
    if (toolbox_left()) {
      channel *c = moving_track->channel;
      int val = ((GdkEventMotion *)ev)->x * 4096;
      int diff;
      while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
      diff = val - ((pos *)c)->subchannel_start;
      set_subchannel_start((pos *)c, val);
      set_subchannel_nb_samples((pos *)c, ((pos *)c)->subchannel_nb_samples - diff);
      update_entry_fields((pos *)c);
    }
    if (toolbox_right()) {
      channel *c = moving_track->channel;
      int val = ((GdkEventMotion *)ev)->x * 4096;
      while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
      val = val - ((pos *)c)->subchannel_start + 1;
      set_subchannel_nb_samples((pos *)c, val);
      update_entry_fields((pos *)c);
    }

    /* this must come after updating stuff */
    update_current_point(widget, ((GdkEventMotion *)ev)->x);
    return TRUE;
  }

  update_current_point(widget, ((GdkEventMotion *)ev)->x);

  return TRUE;
}

int enter_track_window(void *widget, void *ev)
{
  extern void *track_move, *track_left_arrow, *track_right_arrow;
  extern GtkWidget *track_list_right;

  update_current_point(widget, ((GdkEventCrossing *)ev)->x);

  if (widget != track_list_right) {
    if (toolbox_move())
      gdk_window_set_cursor(GTK_WIDGET(((track *)widget)->drawing_area)->window, track_move);
    else if (toolbox_left())
      gdk_window_set_cursor(GTK_WIDGET(((track *)widget)->drawing_area)->window, track_left_arrow);
    else if (toolbox_right())
      gdk_window_set_cursor(GTK_WIDGET(((track *)widget)->drawing_area)->window, track_right_arrow);
    else
      gdk_window_set_cursor(GTK_WIDGET(((track *)widget)->drawing_area)->window, 0);
  }

  return TRUE;
}

int leave_track_window(void *t)
{
/*  extern GtkWidget *track_list_right;*/
  gtk_label_set_text(GTK_LABEL(widget_current_point), "");

#if 0
  if (t != track_list_right) {
    if (toolbox_move())
      gdk_window_set_cursor(GTK_WIDGET(((track *)t)->drawing_area)->window, 0);
  }
#endif

  return TRUE;
}

int track_scroll_event(void *t, void *ev)
{
  extern GtkRange *scrollbar_right_vert;

  /* the 'roulette' of the mouse */
  if (((GdkEventScroll *)ev)->direction == GDK_SCROLL_UP) {
    gtk_range_set_value(scrollbar_right_vert, gtk_range_get_value(scrollbar_right_vert)-20);
    return TRUE;
  }

  if (((GdkEventScroll *)ev)->direction == GDK_SCROLL_DOWN) {
    gtk_range_set_value(scrollbar_right_vert, gtk_range_get_value(scrollbar_right_vert)+20);
    return TRUE;
  }

  return FALSE;
}

int track_button_press_event(void *t, void *ev)
{
  extern GtkWidget *track_list_right;

  if (((GdkEventButton *)ev)->button != 1)
    return FALSE;

  if (toolbox_mark()) {
    int sample;
    double x;
    x = ((GdkEventButton *)ev)->x;
    if (t != (track *)track_list_right) {
      channel *c = ((track *)t)->channel;
      while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
      x += (((pos *)c)->global_start-main_mixer->minimum_sample+4095)/4096;
    }
    sample = x * 4096 + main_mixer->minimum_sample;

    mark_gui_new_with_sample(sample);
    return TRUE;
  }

  if (t == (track *)track_list_right)
    return TRUE;

  if (toolbox_left()) {
    channel *c;
    int val;
    int diff;
    c = ((track *)t)->channel;
    while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
    val = ((GdkEventButton *)ev)->x * 4096;
    diff = val - ((pos *)c)->subchannel_start;
    set_subchannel_start((pos *)c, val);
    set_subchannel_nb_samples((pos *)c, ((pos *)c)->subchannel_nb_samples - diff);
    update_entry_fields((pos *)c);
    is_moving = 1;
    moving_track = t;
    return TRUE;
  }

  if (toolbox_right()) {
    channel *c;
    int val;
    c = ((track *)t)->channel;
    while (c->c.type != CHANNEL_TYPE_POS) c = c->c.subchannel;
    val = ((GdkEventMotion *)ev)->x * 4096 - ((pos *)c)->subchannel_start + 1;
    set_subchannel_nb_samples((pos *)c, val);
    update_entry_fields((pos *)c);
    is_moving = 1;
    moving_track = t;
    return TRUE;
  }

  if (toolbox_move()) {
    is_moving = 1;
    moving_track = t;
    last_position = ((GdkEventButton *)ev)->x_root;
    return TRUE;
  }

  return TRUE;
}

int track_button_release_event(void *t, void *ev)
{
  if (((GdkEventButton *)ev)->button != 1)
    return FALSE;

  if (toolbox_move() || toolbox_left() || toolbox_right()) {
    is_moving = 0;
    return TRUE;
  }

  return FALSE;

}

int marker_track_button_press_event(void *unused, void *ev)
{
  double x = ((GdkEventButton *)ev)->x;

  if (((GdkEventButton *)ev)->button != 1)
    return FALSE;

  main_mixer->next_sample = x * 4096 + main_mixer->minimum_sample;

  return TRUE;
}

void mute_all(void)
{
  int i;

  for (i=0; i<main_mixer->nb_tracks; i++) {
    track *t = main_mixer->tracks[i];
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t->mute_button), TRUE);
  }
}

void mute_none(void)
{
  int i;

  for (i=0; i<main_mixer->nb_tracks; i++) {
    track *t = main_mixer->tracks[i];
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t->mute_button), FALSE);
  }
}
