/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "vol.h"

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "error.h"
#include "vol_gui.h"

static int feed_buffer_short_mono(vol *this, short *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_short_mono(this->c.subchannel, buf, nb_samples, start);

  if (this->vol == 0)
    return 0;

  if (!this->c.subchannel->c.feed_buffer_short_mono(this->c.subchannel, buf, nb_samples, start))
    return 0;

  for (i=0; i<nb_samples; i++) {
    float n = (float)buf[i] * this->vol;
    if (n > 32767) n = 32767;
    else if (n<-32768) n = -32768;
    buf[i] = n;
  }

  return 1;
}

static int feed_buffer_float_mono(vol *this, float *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, buf, nb_samples, start);

  if (this->vol == 0)
    return 0;

  if (!this->c.subchannel->c.feed_buffer_float_mono(this->c.subchannel, buf, nb_samples, start))
   return 0;

  for (i=0; i<nb_samples; i++)
    buf[i] = buf[i] * this->vol;

  return 1;
}

static int feed_buffer_short_stereo(vol *this, short *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, buf, nb_samples, start);

  if (this->vol == 0)
    return 0;

  if (!this->c.subchannel->c.feed_buffer_short_stereo(this->c.subchannel, buf, nb_samples, start))
    return 0;

  for (i=0; i<nb_samples * 2; i++) {
    float n = (float)buf[i] * this->vol;
    if (n > 32767) n = 32767;
    else if (n<-32768) n = -32768;
    buf[i] = n;
  }

  return 1;
}

static int feed_buffer_float_stereo(vol *this, float *buf, int nb_samples, int start)
{
  int i;

  if (this->c.bypass)
    return this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start);

  if (this->vol == 0)
    return 0;

  if (!this->c.subchannel->c.feed_buffer_float_stereo(this->c.subchannel, buf, nb_samples, start))
    return 0;

  for (i=0; i<nb_samples * 2; i++)
    buf[i] = buf[i] * this->vol;

  return 1;
}

static int first_point(vol *this)
{
  return this->c.subchannel->c.first_point(this->c.subchannel);
}

static int last_point(vol *this)
{
  return this->c.subchannel->c.last_point(this->c.subchannel);
}

static void reset(vol *this)
{
  this->c.subchannel->c.reset(this->c.subchannel);
}

vol *new_vol(channel *c)
{
  vol *ret = malloc(sizeof(vol));

  if (!ret)
    return 0;

  memset(ret, 0, sizeof(vol));

  ret->c.type = CHANNEL_TYPE_VOL;

  ret->c.subchannel = c;

  ret->c.feed_buffer_short_mono = (int (*)(void *, short *, int, int))feed_buffer_short_mono;
  ret->c.feed_buffer_float_mono = (int (*)(void *, float *, int, int))feed_buffer_float_mono;
  ret->c.feed_buffer_short_stereo = (int (*)(void *, short *, int, int))feed_buffer_short_stereo;
  ret->c.feed_buffer_float_stereo = (int (*)(void *, float *, int, int))feed_buffer_float_stereo;
  ret->c.first_point = (int (*)(void *))first_point;
  ret->c.last_point = (int (*)(void *))last_point;
  ret->c.reset = (void (*)(void *))reset;

  ret->c.open_gui = (void (*)(void *, int))vol_open_gui;
  ret->c.close_gui = (void (*)(void *))vol_close_gui;

  set_vol(ret, 0);

  return ret;
}

void delete_vol(vol *v)
{
  free(v);
}

void set_vol(vol *p, double v)
{
  while (p && p->c.type != CHANNEL_TYPE_VOL)
    p = (vol *)p->c.subchannel;

  if (!p) {
    reset_error();
    add_error_char("Internal error\n");
    add_error_char(__FILE__);
    add_error_char(":");
    add_error_char(__FUNCTION__);
    add_error_char("\n");
    display_error();
    return;
  }

  p->vol_dB = v;
  p->vol = pow(10, v/20);
}
