/*
 * songs - a little tool to record and mix audio files.
 *
 * public domain
 *
 * http://sed.free.fr/songs
 */

#include "vol_gui.h"

#include <gtk/gtk.h>

#include "error.h"
#include "vol.h"
#include "main_gui.h"
#include "effects_gui.h"

static void scale_change_event(vol *p)
{
  modify = 1;

  set_vol(p, gtk_range_get_value(GTK_RANGE(p->scale)));
}

static void destroy_event(vol *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  effects_gui_force_redraw();
}

void vol_open_gui(vol *p, int track_number)
{
  GtkWidget *win;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *scale;

  if (p->c.gui_on)
    return;

  /* create gui */
  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(win), box);

  /* set track number */
  /* user error as buffer */
  reset_error();
  add_error_char("# ");
  add_error_int(track_number+1);
  label = gtk_label_new(get_error());

  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

  /* set scale */
  scale = gtk_vscale_new_with_range(-60, 40, 0.01);
  gtk_range_set_increments(GTK_RANGE(scale), 0.01, 0.1);
  gtk_range_set_inverted(GTK_RANGE(scale), TRUE);
  gtk_range_set_value(GTK_RANGE(scale), p->vol_dB);
  gtk_widget_set_size_request(scale, 30, 200);
  gtk_scale_set_digits(GTK_SCALE(scale), 2);
  gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_BOTTOM);
  gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_CONTINUOUS);

  gtk_box_pack_start(GTK_BOX(box), scale, TRUE, TRUE, 0);

  /* callback */
  g_signal_connect_swapped(scale, "value-changed", G_CALLBACK(scale_change_event), p);
  g_signal_connect_swapped(win, "destroy", G_CALLBACK(destroy_event), p);

  gtk_widget_show_all(win);

  /* done, let's save data stuff */
  p->c.gui_on = 1;
  p->win = win;
  p->scale = scale;
  p->c.label = label;
}

void vol_close_gui(vol *p)
{
  if (!p->c.gui_on)
    return;

  p->c.gui_on = 0;

  p->c.label = 0;

  gtk_widget_destroy(p->win);
}
